@@ -1,4 +1,4 @@
-/* $Id: copybox.c,v 1.1.1.1 2001/05/12 23:01:36 cegger Exp $
+/* $Id: copybox.c,v 1.4 2005/07/31 09:58:44 cegger Exp $
 ******************************************************************************
 
    LibGGI - Millennium II acceleration for fbdev target
@@ -32,13 +32,13 @@ int GGI_m2164w_copybox(ggi_visual *vis, int x, int y, int w, int h,
 		       int dstx, int dsty)
 {
 	struct m2164w_priv *priv = M2164W_PRIV(vis);
-	volatile uint8 *mmioaddr = FBDEV_PRIV(vis)->mmioaddr;
+	volatile uint8_t *mmioaddr = FBDEV_PRIV(vis)->mmioaddr;
 	int virtx = LIBGGI_VIRTX(vis);
 	int yadd = vis->w_frame_num * LIBGGI_VIRTY(vis);
-	sint32 ar5 = virtx;
-	sint32 begin, end;
-	uint32 sgn = 0;
-	uint32 dwgctl;
+	int32_t ar5 = virtx;
+	int32_t begin, end;
+	uint32_t sgn = 0;
+	uint32_t dwgctl;
 
 #define COPY_LEFT	1
 #define COPY_UP		4
@@ -83,8 +83,10 @@ int GGI_m2164w_copybox(ggi_visual *vis, int x, int y, int w, int h,
 	mga_out32(mmioaddr, RS18(end), AR0);
 	mga_out32(mmioaddr, RS24(begin), AR3);
 	mga_out32(mmioaddr, RS18(ar5), AR5);
-	mga_out32(mmioaddr, (RS16(dstx + w) << 16) | RS16(dstx), FXBNDRY);
-	mga_out32(mmioaddr, (RS16(dsty) << 16) | RS16(h), YDSTLEN | EXECUTE);
+	mga_out32(mmioaddr, (unsigned)(RS16(dstx + w) << 16) | RS16(dstx), 
+		  FXBNDRY);
+	mga_out32(mmioaddr, (unsigned)(RS16(dsty) << 16) | RS16(h), 
+		  YDSTLEN | EXECUTE);
 
 	vis->accelactive = 1;
 
