@@ -1,4 +1,4 @@
-/* $Id: hline.c,v 1.1.1.1 2001/05/12 23:01:36 cegger Exp $
+/* $Id: hline.c,v 1.3 2005/07/30 11:39:57 cegger Exp $
 ******************************************************************************
 
    LibGGI - Millennium II acceleration for fbdev target
@@ -31,9 +31,9 @@
 int GGI_m2164w_drawhline(ggi_visual *vis, int x, int y, int w)
 {
 	struct m2164w_priv *priv = M2164W_PRIV(vis);
-	volatile uint8 *mmioaddr = FBDEV_PRIV(vis)->mmioaddr;
+	volatile uint8_t *mmioaddr = FBDEV_PRIV(vis)->mmioaddr;
 	int yadd = vis->w_frame_num * LIBGGI_VIRTY(vis);
-	uint32 dwgctl;
+	uint32_t dwgctl;
 
 	y += yadd;
 	y = RS16(y) << 16;
@@ -51,8 +51,8 @@ int GGI_m2164w_drawhline(ggi_visual *vis, int x, int y, int w)
 		mga_waitfifo(mmioaddr, 2);
 	}
 	/* y has been shifted above */
-	mga_out32(mmioaddr, RS16(x) | y, XYSTRT);
-	mga_out32(mmioaddr, RS16(x + w-1) | y, XYEND | EXECUTE);
+	mga_out32(mmioaddr, (unsigned)RS16(x) | y, XYSTRT);
+	mga_out32(mmioaddr, (unsigned)RS16(x + w-1) | y, XYEND | EXECUTE);
 
 	vis->accelactive = 1;
 
