@@ -1,4 +1,4 @@
-/* $Id: crossblit.c,v 1.10 2005/07/30 11:40:00 cegger Exp $
+/* $Id$
 ******************************************************************************
 
    24-bpp linear direct-access framebuffer renderer for LibGGI:
@@ -138,11 +138,103 @@ cb4to24(ggi_visual *src, int sx, int sy, int w, int h,
 
 	dststride -= w*3;
 
+	if (((sx ^ w) & 1) && (GT_SUBSCHEME(LIBGGI_GT(src)) & GT_SUB_HIGHBIT_RIGHT)) {
+		for (; h > 0; h--) {
+			uint8_t  *srcpb = srcp;
+			
+			int i = (w + 7) / 8;
+
+			/* Unroll manually. */
+			switch (w & 0x7) {
+			default:
+				for (; i > 0; i--) {
+				case 0:
+				  put24(dstp, conv_tab[(*srcpb & 0xf0) >> 4]);
+				  srcpb++;
+				  dstp += 3;
+				case 7:
+				  put24(dstp, conv_tab[(*srcpb & 0x0f)]);
+				  dstp += 3;
+				case 6:
+				  put24(dstp, conv_tab[(*srcpb & 0xf0) >> 4]);
+				  srcpb++;
+				  dstp += 3;
+				case 5:
+				  put24(dstp, conv_tab[(*srcpb & 0x0f)]);
+				  dstp += 3;
+				case 4:
+				  put24(dstp, conv_tab[(*srcpb & 0xf0) >> 4]);
+				  srcpb++;
+				  dstp += 3;
+				case 3:
+				  put24(dstp, conv_tab[(*srcpb & 0x0f)]);
+				  dstp += 3;
+				case 2:
+				  put24(dstp, conv_tab[(*srcpb & 0xf0) >> 4]);
+				  srcpb++;
+				  dstp += 3;
+				case 1:
+				  put24(dstp, conv_tab[(*srcpb & 0x0f)]);
+				  dstp += 3;
+				}
+			}
+			
+			srcp += srcstride;
+			dstp += dststride;
+		}
+		return;
+	}
+	if (GT_SUBSCHEME(LIBGGI_GT(src)) & GT_SUB_HIGHBIT_RIGHT) {
+		for (; h > 0; h--) {
+			uint8_t *srcpb = srcp;
+			
+			int i = (w + 7) / 8;
+
+			/* Unroll manually. */			
+			switch (w & 0x7) {
+			default:
+				for (; i > 0; i--) {
+				case 0:
+				  put24(dstp, conv_tab[(*srcpb & 0x0f)]);
+				  dstp += 3;
+				case 7:
+				  put24(dstp, conv_tab[(*srcpb & 0xf0) >> 4]);
+				  srcpb++;
+				  dstp += 3;
+				case 6:
+				  put24(dstp, conv_tab[(*srcpb & 0x0f)]);
+				  dstp += 3;
+				case 5:
+				  put24(dstp, conv_tab[(*srcpb & 0xf0) >> 4]);
+				  srcpb++;
+				  dstp += 3;
+				case 4:
+				  put24(dstp, conv_tab[(*srcpb & 0x0f)]);
+				  dstp += 3;
+				case 3:
+				  put24(dstp, conv_tab[(*srcpb & 0xf0) >> 4]);
+				  srcpb++;
+				  dstp += 3;
+				case 2:
+				  put24(dstp, conv_tab[(*srcpb & 0x0f)]);
+				  dstp += 3;
+				case 1:
+				  put24(dstp, conv_tab[(*srcpb & 0xf0) >> 4]);
+				  srcpb++;
+				  dstp += 3;
+				}
+			}
+			
+			srcp += srcstride;
+			dstp += dststride;
+		}
+		return;
+	}
 	if ((sx ^ w) & 1) {
 		for (; h > 0; h--) {
 			uint8_t  *srcpb = srcp;
 			
-			int i = w / 8;
+			int i = (w + 7) / 8;
 
 			/* Unroll manually. */
 			switch (w & 0x7) {
@@ -187,8 +279,7 @@ cb4to24(ggi_visual *src, int sx, int sy, int w, int h,
 		for (; h > 0; h--) {
 			uint8_t *srcpb = srcp;
 			
-			int i = w / 8;
-
+			int i = (w + 7) / 8;
 
 			/* Unroll manually. */			
 			switch (w & 0x7) {
@@ -263,7 +354,7 @@ cb8to24(ggi_visual *src, int sx, int sy, int w, int h,
 	dststride -= w*3;
 
 	for (; h > 0; h--) {
-		int i = w / 8;
+		int i = (w + 7) / 8;
 
 		/* We don't believe in the optimizing capabilities of the
 		 * compiler hence unroll manually.
@@ -779,8 +870,8 @@ static inline void cb32to24(ggi_visual *src, int sx, int sy, int w, int h,
 
 	while (stoprow > dstp) {
 		uint8_t *stopcol;
-		
-		stopcol = dstp + w;
+
+		stopcol = dstp + w * 3;
 		while (stopcol > dstp) {
 			ggi_pixel tmp, cache;
 
@@ -948,6 +1039,11 @@ int GGI_lin24_crossblit(ggi_visual *src, int sx, int sy, int w, int h,
 		else goto fallback;
 		return 0;
 	case 16:
+		if (GT_SCHEME(LIBGGI_GT(src)) == GT_TRUECOLOR)
+		  cb16to24(src, sx, sy, w, h, dst, dx, dy);
+		else goto fallback;
+		return 0;
+	case 24:
 		if (!dst->w_frame->buffer.plb.pixelformat->stdformat) 
 		  goto notsame;
 		if (dst->w_frame->buffer.plb.pixelformat->stdformat !=
@@ -956,11 +1052,6 @@ int GGI_lin24_crossblit(ggi_visual *src, int sx, int sy, int w, int h,
 		crossblit_same(src, sx, sy, w, h, dst, dx, dy);
 		return 0;
 	notsame:
-		if (GT_SCHEME(LIBGGI_GT(src)) == GT_TRUECOLOR)
-		  cb16to24(src, sx, sy, w, h, dst, dx, dy);
-		else goto fallback;
-		return 0;
-	case 24:
 		if (GT_SCHEME(LIBGGI_GT(src)) == GT_TRUECOLOR)
 		  cb24to24(src, sx, sy, w, h, dst, dx, dy);
 		else goto fallback;
