@@ -1,4 +1,4 @@
-/* $Id: crossblit.c,v 1.13 2005/07/30 11:40:01 cegger Exp $
+/* $Id$
 ******************************************************************************
 
    32-bpp linear direct-access framebuffer renderer for LibGGI:
@@ -89,9 +89,6 @@ crossblit_same(ggi_visual *src, int sx, int sy, int w, int h,
 	srcp = (uint8_t*)LIBGGI_CURREAD(src)  + srcstride*sy + sx*4;
 	dstp = (uint8_t*)LIBGGI_CURWRITE(dst) + dststride*dy + dx*4;
 
-	srcstride -= w*4;
-	dststride -= w*4;
-
 	while (h--) {
 		tmpw = w;
 		srcp32 = (uint32_t *)srcp;
@@ -134,12 +131,82 @@ cb4to32(ggi_visual *src, int sx, int sy, int w, int h,
 	srcp = (uint8_t*)LIBGGI_CURREAD(src)  + srcstride*sy + sx/2;
 	dstp = (uint8_t*)LIBGGI_CURWRITE(dst) + dststride*dy + dx*4;
 
+	if (((sx ^ w) & 1) && (GT_SUBSCHEME(LIBGGI_GT(src)) & GT_SUB_HIGHBIT_RIGHT)) {
+		for (; h > 0; --h) {
+			uint32_t *dstpw = (uint32_t*) dstp;
+			uint8_t  *srcpb = srcp;
+
+			int i = (w + 7) / 8;
+
+			/* Unroll manually. */
+			switch (w & 0x7) {
+			default:
+				for (; i > 0; --i) {
+				case 0:
+				*dstpw++ = conv_tab[(*srcpb++ & 0xf0) >> 4];
+				case 7:
+				*dstpw++ = conv_tab[(*srcpb & 0x0f)];
+				case 6:
+				*dstpw++ = conv_tab[(*srcpb++ & 0xf0) >> 4];
+				case 5:
+				*dstpw++ = conv_tab[(*srcpb & 0x0f)];
+				case 4:
+				*dstpw++ = conv_tab[(*srcpb++ & 0xf0) >> 4];
+				case 3:
+				*dstpw++ = conv_tab[(*srcpb & 0x0f)];
+				case 2:
+				*dstpw++ = conv_tab[(*srcpb++ & 0xf0) >> 4];
+				case 1:
+				*dstpw++ = conv_tab[(*srcpb & 0x0f)];
+				}
+			}
+
+			srcp += srcstride;
+			dstp += dststride;
+		}
+		return;
+	}
+	if (GT_SUBSCHEME(LIBGGI_GT(src)) & GT_SUB_HIGHBIT_RIGHT) {
+		for (; h > 0; --h) {
+			uint32_t *dstpw = (uint32_t*) dstp;
+			uint8_t  *srcpb = srcp;
+
+			int i = (w + 7) / 8;
+
+			/* Unroll manually. */			
+			switch (w & 0x7) {
+			default:
+				for (; i > 0; --i) {
+				case 0:
+				*dstpw++ = conv_tab[(*srcpb & 0x0f)];
+				case 7:
+				*dstpw++ = conv_tab[(*srcpb++ & 0xf0) >> 4];
+				case 6:
+				*dstpw++ = conv_tab[(*srcpb & 0x0f)];
+				case 5:
+				*dstpw++ = conv_tab[(*srcpb++ & 0xf0) >> 4];
+				case 4:
+				*dstpw++ = conv_tab[(*srcpb & 0x0f)];
+				case 3:
+				*dstpw++ = conv_tab[(*srcpb++ & 0xf0) >> 4];
+				case 2:
+				*dstpw++ = conv_tab[(*srcpb & 0x0f)];
+				case 1:
+				*dstpw++ = conv_tab[(*srcpb++ & 0xf0) >> 4];
+				}
+			}
+
+			srcp += srcstride;
+			dstp += dststride;
+		}
+		return;
+	}
 	if ((sx ^ w) & 1) {
 		for (; h > 0; h--) {
 	    		uint32_t *dstpw = (uint32_t*) dstp;
 			uint8_t  *srcpb = srcp;
 			
-			int i = w / 8;
+			int i = (w + 7) / 8;
 			
 			/* Unroll manually. */
 			switch (w & 0x7) {
@@ -173,7 +240,7 @@ cb4to32(ggi_visual *src, int sx, int sy, int w, int h,
 			uint32_t *dstpw = (uint32_t*) dstp;
 			uint8_t  *srcpb = srcp;
 			
-			int i = w / 8;
+			int i = (w + 7) / 8;
 
 			/* Unroll manually. */			
 			switch (w & 0x7) {
@@ -235,7 +302,7 @@ cb8to32(ggi_visual *src, int sx, int sy, int w, int h,
 	for (; h > 0; h--) {
 		uint32_t *dstpw = (uint32_t*) dstp;
 		uint8_t  *srcpb = (uint8_t*)  srcp;
-		int i = w / 8;
+		int i = (w + 7) / 8;
 
 		/* We don't believe in the optimizing capabilities of the
 		 * compiler hence unroll manually.
@@ -969,14 +1036,6 @@ int GGI_lin32_crossblit(ggi_visual *src, int sx, int sy, int w, int h,
 		else goto fallback;
 		return 0;
 	case 16:
-		if (!dst->w_frame->buffer.plb.pixelformat->stdformat) 
-		  goto notsame;
-		if (dst->w_frame->buffer.plb.pixelformat->stdformat !=
-		    src->r_frame->buffer.plb.pixelformat->stdformat) 
-		  goto notsame;
-		crossblit_same(src, sx, sy, w, h, dst, dx, dy);
-		return 0;
-	notsame:
 		if (GT_SCHEME(LIBGGI_GT(src)) == GT_TRUECOLOR)
 		  cb16to32(src, sx, sy, w, h, dst, dx, dy);
 		else goto fallback;
@@ -987,6 +1046,14 @@ int GGI_lin32_crossblit(ggi_visual *src, int sx, int sy, int w, int h,
 		else goto fallback;
 		return 0;
 	case 32:
+		if (!dst->w_frame->buffer.plb.pixelformat->stdformat) 
+		  goto notsame;
+		if (dst->w_frame->buffer.plb.pixelformat->stdformat !=
+		    src->r_frame->buffer.plb.pixelformat->stdformat) 
+		  goto notsame;
+		crossblit_same(src, sx, sy, w, h, dst, dx, dy);
+		return 0;
+	notsame:
 		if (GT_SCHEME(LIBGGI_GT(src)) == GT_TRUECOLOR)
 		  cb32to32(src, sx, sy, w, h, dst, dx, dy);
 		else goto fallback;
