@@ -1,4 +1,4 @@
-/* $Id: box.c,v 1.1.1.1 2001/05/12 23:01:45 cegger Exp $
+/* $Id: box.c,v 1.5 2005/07/30 11:40:02 cegger Exp $
 ******************************************************************************
 
    Graphics library for GGI. Boxes.
@@ -32,23 +32,23 @@
 
 int GGI_lin8_drawbox(ggi_visual *vis, int x, int y, int w, int h)
 {
-	uint8 *dest;
+	uint8_t *dest;
 	int destwidth = LIBGGI_FB_W_STRIDE(vis);
-	uint8 col = LIBGGI_GC_FGCOLOR(vis);
+	uint8_t col = LIBGGI_GC_FGCOLOR(vis);
 
 	LIBGGICLIP_XYWH(vis, x, y, w, h);
 	PREPARE_FB(vis);
 
-	dest = (uint8 *)LIBGGI_CURWRITE(vis) + (y*destwidth + x);
+	dest = (uint8_t *)LIBGGI_CURWRITE(vis) + (y*destwidth + x);
 
 	/* Optimized full-width case */
 	if (w == destwidth && x == 0) {
-		memset(dest, col, w*h);
+		memset(dest, col, (size_t)(w*h));
 		return 0;
 	}
 	
 	while (h > 0) {
-		memset(dest, col, w);
+		memset(dest, col, (size_t)(w));
 		dest += destwidth;
 		h--;
 	}
@@ -56,26 +56,26 @@ int GGI_lin8_drawbox(ggi_visual *vis, int x, int y, int w, int h)
 }
 
 
-int GGI_lin8_putbox(ggi_visual *vis, int x, int y, int w, int h, void *buffer)
+int GGI_lin8_putbox(ggi_visual *vis, int x, int y, int w, int h, const void *buffer)
 {
-	uint8 *src = buffer;
-	uint8 *dest;
+	const uint8_t *src = buffer;
+	uint8_t *dest;
 	int srcwidth = w;
 	int destwidth = LIBGGI_FB_W_STRIDE(vis);
 
-	LIBGGICLIP_PUTBOX(vis,x,y,w,h,src,srcwidth, );
+	LIBGGICLIP_PUTBOX(vis,x,y,w,h,src,srcwidth, *1);
 	PREPARE_FB(vis);
 
-	dest = (uint8 *)LIBGGI_CURWRITE(vis) + (y*destwidth + x);
+	dest = (uint8_t *)LIBGGI_CURWRITE(vis) + (y*destwidth + x);
 
 	/* Optimized full-width case */
 	if (w == destwidth && x == 0) {
-		memcpy(dest, src, w*h);
+		memcpy(dest, src, (size_t)(w*h));
 		return 0;
 	}
 	
 	while (h > 0) {
-		memcpy(dest, src, w);
+		memcpy(dest, src, (size_t)(w));
 		dest += destwidth;
 		src += srcwidth;
 		h--;
