@@ -1,4 +1,4 @@
-/* $Id: visual.c,v 1.2 2002/09/08 21:37:45 soyt Exp $
+/* $Id: visual.c,v 1.14 2005/09/19 18:46:40 cegger Exp $
 ******************************************************************************
 
    AAlib target for GGI.
@@ -28,13 +28,27 @@
 */
 
 #include <stdlib.h>
+#include <string.h>
 
 #include "config.h"
-#include <ggi/internal/ggi-dl.h>
 #include <ggi/display/aa.h>
+#include <ggi/internal/ggi_debug.h>
 
 
-void _GGI_aa_freedbs(ggi_visual *vis) {
+static gg_option optlist[] =
+{
+	{ "fastrender", "no" }
+};
+
+
+#define OPT_FASTRENDER	0
+
+#define NUM_OPTS	(sizeof(optlist)/sizeof(gg_option))
+
+
+
+void _GGI_aa_freedbs(ggi_visual *vis)
+{
 	int i;
 
 	for (i=LIBGGI_APPLIST(vis)->num-1; i >= 0; i--) {
@@ -45,11 +59,11 @@ void _GGI_aa_freedbs(ggi_visual *vis) {
 
 static int GGIclose(ggi_visual *vis, struct ggi_dlhandle *dlh)
 {
-	ggi_aa_priv *priv;
+	ggi_aa_priv *priv = AA_PRIV(vis);
 
 	_GGI_aa_freedbs(vis);
 
-	if ((priv = LIBGGI_PRIVATE(vis)) != NULL) {
+	if (priv != NULL) {
 		if (priv->context) {
 			aa_uninitmouse(priv->context);
 			aa_uninitkbd(priv->context);
@@ -57,7 +71,7 @@ static int GGIclose(ggi_visual *vis, struct ggi_dlhandle *dlh)
 		}
 		free(priv->opmansync);		
 		ggLockDestroy(priv->aalock);
-		free(LIBGGI_PRIVATE(vis));
+		free(priv);
 	}
 
 	free(LIBGGI_GC(vis));
@@ -66,15 +80,16 @@ static int GGIclose(ggi_visual *vis, struct ggi_dlhandle *dlh)
 }
 
 static int GGIopen(ggi_visual *vis, struct ggi_dlhandle *dlh,
-		   const char *args, void *argptr, uint32 *dlret)
+		   const char *args, void *argptr, uint32_t *dlret)
 {
 	ggi_aa_priv *priv;
 	void *lock;
 	int err = GGI_ENOMEM;
-	gg_option optlist[] = {
-		{ "fastrender", "" }};
+	gg_option options[NUM_OPTS];
 	
-	GGIDPRINT_LIBS("display-aa: Starting\n");
+	DPRINT_LIBS("display-aa: Starting\n");
+
+	memcpy(options, optlist, sizeof(options));
 	
 	/* Get options from environment variable AAOPTS */
 	if (!aa_parseoptions(NULL, NULL, NULL, NULL)) {
@@ -110,16 +125,16 @@ static int GGIopen(ggi_visual *vis, struct ggi_dlhandle *dlh,
 	priv->haverelease = 0;
 
 	if (args) {
-		args = ggParseOptions((char *)args, optlist,
-					sizeof(optlist)/sizeof(gg_option));
-		if(!args) {
+		args = ggParseOptions((char *)args, options, NUM_OPTS);
+		if (!args) {
 			fprintf(stderr, "display-aa: error in arguments\n");
 		}
 
-		priv->fastrender = (*args && optlist[0].result[0]=='y');
+		priv->fastrender = (*args && options[OPT_FASTRENDER].result[0] == 'y');
 	}
 
-	err = _ggiAddDL(vis, "helper-mansync", NULL, priv->opmansync, 0);
+	err = _ggiAddDL(vis, _ggiGetConfigHandle(),
+			"helper-mansync", NULL, priv->opmansync, 0);
 	if (err) {
 		fprintf(stderr, 
 			"display-aa: Cannot load required helper-mansync!\n");
@@ -129,19 +144,22 @@ static int GGIopen(ggi_visual *vis, struct ggi_dlhandle *dlh,
 	LIBGGI_PRIVATE(vis) = priv;
 
 	MANSYNC_init(vis);
+	if (!(LIBGGI_FLAGS(vis) & GGIFLAG_ASYNC)) {
+		MANSYNC_start(vis);
+	}
 
-	{
+	do {
 		gii_input *inp;
-		GGIDPRINT_MISC("display-aa: gii starting\n");
+		DPRINT_MISC("display-aa: gii starting\n");
 
 		/* First allocate a new gii_input descriptor. */
 		inp = _giiInputAlloc();
 		if (inp == NULL) {
-			GGIDPRINT_MISC("display-aa: _giiInputAlloc failed\n");
+			DPRINT_MISC("display-aa: _giiInputAlloc failed\n");
 			GGIclose(vis, dlh);
 			return GGI_ENOMEM;
 		}
-		GGIDPRINT_MISC("display-aa: gii inp=%p\n",inp);
+		DPRINT_MISC("display-aa: gii inp=%p\n",inp);
 
 		/* Now fill in the blanks. */
 		inp->priv = priv; /* We need that in GII_aa_poll() */
@@ -161,9 +179,10 @@ static int GGIopen(ggi_visual *vis, struct ggi_dlhandle *dlh,
 		/* Now join the new event source in. */
 		vis->input = giiJoinInputs(vis->input,inp);
 		
-		GGIDPRINT_MISC("display-aa: input joined into %p\n",
+		DPRINT_MISC("display-aa: input joined into %p\n",
 			       vis->input);
-	}
+	} while(0);
+
 
 	/* Has mode management */
 	vis->opdisplay->flush=GGI_aa_flush;
@@ -196,17 +215,20 @@ static int GGIexit(ggi_visual *vis, struct ggi_dlhandle *dlh)
 }
 
 
+EXPORTFUNC
+int GGIdl_aa(int func, void **funcptr);
+
 int GGIdl_aa(int func, void **funcptr)
 {
 	switch (func) {
 	case GGIFUNC_open:
-		*funcptr = GGIopen;
+		*funcptr = (void *)GGIopen;
 		return 0;
 	case GGIFUNC_exit:
-		*funcptr = GGIexit;
+		*funcptr = (void *)GGIexit;
 		return 0;
 	case GGIFUNC_close:
-		*funcptr = GGIclose;
+		*funcptr = (void *)GGIclose;
 		return 0;
 	default:
 		*funcptr = NULL;
