@@ -1,10 +1,11 @@
-/* $Id: mode.c,v 1.10 2002/10/27 18:26:23 skids Exp $
+/* $Id: mode.c,v 1.28 2005/09/19 18:46:41 cegger Exp $
 ******************************************************************************
 
    Display-FBDEV
 
-   Copyright (C) 1998 Andrew Apted		[andrew@ggi-project.org]
+   Copyright (C) 1998      Andrew Apted		[andrew@ggi-project.org]
    Copyright (C) 1999-2000 Marcus Sundberg	[marcus@ggi-project.org]
+   Copyright (C) 2005      Joseph Crayne	[oh.hello.joe@gmail.com]
 
    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
@@ -37,8 +38,8 @@
 #include <linux/fb.h>
 
 #include "config.h"
-#include <ggi/internal/ggi-dl.h>
 #include <ggi/display/fbdev.h>
+#include <ggi/internal/ggi_debug.h>
 
 #include "../common/gt-auto.inc"
 
@@ -50,8 +51,19 @@
 #define FB_KLUDGE_FONTX	 8
 #define FB_KLUDGE_FONTY	16
 
-static int
-do_checkmode(ggi_visual *vis, ggi_mode *mode, struct fb_var_screeninfo *var);
+extern void GGI_fbdev_color0(ggi_visual *vis);
+extern void GGI_fbdev_color_free(ggi_visual *vis);
+extern void GGI_fbdev_color_setup(ggi_visual *vis);
+int GGI_fbdev_mode_reset(ggi_visual *vis);
+
+#include <ggi/display/modelist.h>
+
+#define WANT_GENERIC_CHECKMODE
+#include "../common/modelist.inc"
+
+static 
+int _GGI_fbdev_do_checkmode(ggi_visual *vis, ggi_mode *mode,
+                            struct ggi_fbdev_timing **timing_);
 
 static void _GGI_free_dbs(ggi_visual *vis) 
 {
@@ -74,18 +86,18 @@ static void clear_fbmem(void *mem, unsigned long len)
 }
 
 
-int GGI_fbdev_kgicommand(ggi_visual *vis,int cmd,void *args)
+int GGI_fbdev_kgicommand(ggi_visual *vis, int cmd,void *args)
 {
-	return fbdev_doioctl(vis, cmd, args);
+	return fbdev_doioctl(vis, (unsigned)cmd, args);
 }
         
 
 int GGI_fbdev_getapi(ggi_visual *vis, int num, char *apiname, char *arguments)
 {
-	ggi_fbdev_priv *priv = LIBGGI_PRIVATE(vis);
+	ggi_fbdev_priv *priv = FBDEV_PRIV(vis);
 	int size = GT_SIZE(LIBGGI_GT(vis));
 
-	strcpy(arguments, "");
+	*arguments = '\0';
 
 	switch(num) {
 
@@ -102,12 +114,12 @@ int GGI_fbdev_getapi(ggi_visual *vis, int num, char *apiname, char *arguments)
 			sprintf(apiname, "generic-text-%d", size);
 			return 0;
 		} 
-		
+
 		if (priv->fix.type == FB_TYPE_PLANES) {
 			strcpy(apiname, "generic-planar");
 			return 0;
 		}
-		
+
 		if (priv->fix.type == FB_TYPE_INTERLEAVED_PLANES) {
 			sprintf(apiname, "generic-%s",
 				(priv->fix.type_aux == 2) ? 
@@ -123,7 +135,7 @@ int GGI_fbdev_getapi(ggi_visual *vis, int num, char *apiname, char *arguments)
 		   a different generic renderer, e.g. if the driver
 		   in question wants a generic-linear-4r instead of 
 		   a generic-linear-4r */
-	  	if (GT_SCHEME(LIBGGI_GT(vis)) == GT_TEXT) {
+		if (GT_SCHEME(LIBGGI_GT(vis)) == GT_TEXT) {
 			sprintf(apiname, "fb-generic-%2.2x-text-%d", 
 				priv->orig_fix.accel, size);
 			return 0;
@@ -147,20 +159,20 @@ int GGI_fbdev_getapi(ggi_visual *vis, int num, char *apiname, char *arguments)
 				priv->orig_fix.accel, size);
 		return 0;
 		break;
-		
+
 	case 5:
-	  	if (GT_SCHEME(LIBGGI_GT(vis)) == GT_TEXT) {
+		if (GT_SCHEME(LIBGGI_GT(vis)) == GT_TEXT) {
 			sprintf(apiname, "fb-accel-%2.2x-text-%d", 
 				priv->orig_fix.accel, size);
 			return 0;
 		} 
-		
+
 		if (priv->fix.type == FB_TYPE_PLANES) {
 			sprintf(apiname, "fb-accel-%2.2x-planar",
 				priv->orig_fix.accel);
 			return 0;
 		}
-		
+
 		if (priv->fix.type == FB_TYPE_INTERLEAVED_PLANES) {
 			sprintf(apiname, "fb-accel-%2.2x-%s",
 				priv->orig_fix.accel ,
@@ -175,7 +187,7 @@ int GGI_fbdev_getapi(ggi_visual *vis, int num, char *apiname, char *arguments)
 		break;
 	}
 
-	return -1;
+	return GGI_ENOMATCH;
 }
 
 
@@ -203,103 +215,10 @@ ggimode2var(ggi_visual *vis, ggi_mode *mode, struct fb_var_screeninfo *var)
 	}
 }
 
-
 static void
-var2ggimode(ggi_visual *vis, struct fb_var_screeninfo *var, ggi_mode *mode,
-	    int frames)
+timing2var( struct ggi_fbdev_timing *timing, struct fb_var_screeninfo *var,
+		int frames )
 {
-	mode->visible.x = var->xres;
-	mode->visible.y = var->yres;
-	mode->virt.x = var->xres_virtual;
-	mode->virt.y = var->yres_virtual;
-	if (frames) {
-		mode->virt.y /= frames;
-	}
-	if (var->bits_per_pixel == 0) {
-		mode->graphtype = GT_TEXT16;
-		return;
-	}
-	GT_SETSIZE(mode->graphtype, var->bits_per_pixel);
-	if (var->grayscale) {
-		GT_SETSCHEME(mode->graphtype, GT_GREYSCALE);
-		GT_SETDEPTH(mode->graphtype, var->bits_per_pixel);
-	} else if (var->red.length + var->green.length + var->blue.length
-		   > var->bits_per_pixel) {
-		GT_SETSCHEME(mode->graphtype, GT_PALETTE);
-		GT_SETDEPTH(mode->graphtype, var->bits_per_pixel);
-	} else {
-		GT_SETSCHEME(mode->graphtype, GT_TRUECOLOR);
-		GT_SETDEPTH(mode->graphtype, var->red.length
-			    + var->green.length + var->blue.length);
-	}
-}
-
-
-#define SKIPWHITE(ptr) while(*(ptr)==' '||*(ptr)=='\t'||*(ptr)=='\n'){(ptr)++;}
-
-static int
-get_timing(ggi_visual *vis, struct fb_var_screeninfo *var, int frames)
-{
-	ggi_fbdev_priv *priv = LIBGGI_PRIVATE(vis);
-	struct ggi_fbdev_timing *timing, *besttiming;
-	int ret = 1;
-
-	/* Scan for the wanted mode.
-	 */
-	besttiming = NULL;
-	timing = priv->timings;
-	while (timing) {
-		if ((timing->xres == var->xres || var->xres == 0) &&
-		    (timing->yres == var->yres || var->yres == 0)) {
-			if ((timing->bits_per_pixel == var->bits_per_pixel ||
-			     var->bits_per_pixel == FB_BPP_AUTO) &&
-			    (timing->xres_virtual == var->xres_virtual ||
-			     var->xres_virtual == 0)) {
-				/* yres_virtual doesn't matter. */
-				GGIDPRINT_MODE("display-fbdev: found exact match.\n");
-				ret = 0;
-				goto found_mode;
-			}
-			if (timing->bits_per_pixel == var->bits_per_pixel ||
-			    var->bits_per_pixel == FB_BPP_AUTO) {
-				GGIDPRINT_MODE("display-fbdev: found match with xres_virtual = %d.\n",
-					       timing->xres_virtual);
-				ret = 0;
-				besttiming = timing;
-			} else if (besttiming == NULL || ret == 1) {
-				GGIDPRINT_MODE("display-fbdev: found first match (exact res) %dx%d.\n",
-					       timing->xres, timing->yres);
-				ret = 0;
-				besttiming = timing;
-			}
-		} else if ((timing->xres >= var->xres || var->xres == 0) &&
-			   (timing->yres >= var->yres || var->yres == 0)) {
-			if (besttiming == NULL) {
-				GGIDPRINT_MODE("display-fbdev: found first match (bigger res) %dx%d.\n",
-					       timing->xres, timing->yres);
-				ret = 1;
-				besttiming = timing;
-			} else if (timing->xres <= besttiming->xres &&
-				   timing->yres <= besttiming->yres &&
-				   ret == 1) {
-				GGIDPRINT_MODE("display-fbdev: found better match (bigger res) %dx%d.\n",
-					       timing->xres, timing->yres);
-				ret = 1;
-				besttiming = timing;
-			}
-		}
-		timing = timing->next;
-	}
-	if (besttiming) {
-		timing = besttiming;
-		GGIDPRINT_MODE("display-fbdev: Using best match %dx%d.\n",
-			       timing->xres, timing->yres);
-		goto found_mode;
-	}
-
-	return GGI_ENOTFOUND;
-
-  found_mode:
 	var->xres = timing->xres;
 	var->yres = timing->yres;
 	if (var->xres_virtual == 0) var->xres_virtual = timing->xres_virtual;
@@ -317,18 +236,68 @@ get_timing(ggi_visual *vis, struct fb_var_screeninfo *var, int frames)
 	var->vsync_len    = timing->vsync_len;
 	var->sync         = timing->sync;
 	var->vmode        = timing->vmode;
-
-	return ret;
 }
 
+static void
+var2ggimode(ggi_visual *vis, struct fb_var_screeninfo *var, ggi_mode *mode,
+	    int frames)
+{
+	mode->visible.x = var->xres;
+	mode->visible.y = var->yres;
+	mode->virt.x = var->xres_virtual;
+	mode->virt.y = var->yres_virtual;
+
+	/* All display-fbdev modes are fullscreen, so we will just
+	 * put the framebuffer's fullscreen dimensions into the
+	 * size field. */
+	if( (signed)var->width >= 0 )
+		mode->size.x = var->width;
+	if( (signed)var->height >= 0 )
+		mode->size.y = var->height;
+	DPRINT_LIBS("var2ggimode stored size (%i, %i)\n", 
+			mode->size.x, mode->size.y );
+
+	if (frames) {
+		mode->virt.y /= frames;
+	}
+	if (var->bits_per_pixel == 0) {
+		mode->graphtype = GT_TEXT16;
+		return;
+	}
+	GT_SETSIZE(mode->graphtype, var->bits_per_pixel);
+	if (var->grayscale) {
+		GT_SETSCHEME(mode->graphtype, GT_GREYSCALE);
+		GT_SETDEPTH(mode->graphtype, var->bits_per_pixel);
+	} else if (var->red.length + var->green.length + var->blue.length
+		   > var->bits_per_pixel)
+	{
+		GT_SETSCHEME(mode->graphtype, GT_PALETTE);
+		GT_SETDEPTH(mode->graphtype, var->bits_per_pixel);
+	} else {
+		GT_SETSCHEME(mode->graphtype, GT_TRUECOLOR);
+		GT_SETDEPTH(mode->graphtype, var->red.length
+			    + var->green.length + var->blue.length);
+	}
+            
+}
+
+
+#define SKIPWHITE(ptr) while(*(ptr)==' '||*(ptr)=='\t'||*(ptr)=='\n'){(ptr)++;}
+
 
 static int do_change_mode(ggi_visual *vis, struct fb_var_screeninfo *var)
 {
-	ggi_fbdev_priv *priv = LIBGGI_PRIVATE(vis);
+	ggi_fbdev_priv *priv = FBDEV_PRIV(vis);
 	ggi_mode *mode = LIBGGI_MODE(vis);
 	ggi_graphtype gt = mode->graphtype;
 	int err;
 
+	/* If we already have set a mode, restore the old palette
+	 * or gamma settings before setting a new one. 
+	 */
+	//GGI_fbdev_color0(vis);
+	GGI_fbdev_color_free(vis);
+
 	/* We need this to set virtual resolution correct */
 	ggimode2var(vis, mode, var);
 
@@ -361,18 +330,22 @@ static int do_change_mode(ggi_visual *vis, struct fb_var_screeninfo *var)
 #endif
 			    priv->fix.visual == FB_VISUAL_TRUECOLOR ||
 			    priv->fix.visual == FB_VISUAL_DIRECTCOLOR)
+			{
 				err = GGI_EFATAL;
+				DPRINT_MODE("display-fbdev: GT_GREYSCALE mode failed\n");
+			}
 			break;
-	
+
 		case GT_PALETTE:
 			if (GT_DEPTH(gt) != priv->var.bits_per_pixel ||
 #ifdef FB_TYPE_TEXT
 			    priv->fix.type == FB_TYPE_TEXT ||
 #endif
 			    priv->fix.visual == FB_VISUAL_TRUECOLOR ||
-			    priv->fix.visual == FB_VISUAL_DIRECTCOLOR) {
+			    priv->fix.visual == FB_VISUAL_DIRECTCOLOR)
+			{
 				err = GGI_EFATAL;
-				GGIDPRINT_MODE("display-fbdev: GT_PALETTE mode failed\n");
+				DPRINT_MODE("display-fbdev: GT_PALETTE mode failed\n");
 			}
 			break;
 
@@ -383,9 +356,10 @@ static int do_change_mode(ggi_visual *vis, struct fb_var_screeninfo *var)
 			    priv->fix.type  == FB_TYPE_TEXT ||
 #endif
 			    (priv->fix.visual != FB_VISUAL_TRUECOLOR &&
-			     priv->fix.visual != FB_VISUAL_DIRECTCOLOR)) {
+			     priv->fix.visual != FB_VISUAL_DIRECTCOLOR))
+			{
 				err = GGI_EFATAL;
-				GGIDPRINT_MODE("display-fbdev: GT_TRUECOLOR mode failed\n");
+				DPRINT_MODE("display-fbdev: GT_TRUECOLOR mode failed\n");
 			}
 			break;
 		
@@ -397,19 +371,19 @@ static int do_change_mode(ggi_visual *vis, struct fb_var_screeninfo *var)
 	}
 	
 	if (err) {
-		GGIDPRINT_MODE("display-fbdev: Santa passed us by :-(\n");
+		DPRINT_MODE("display-fbdev: Santa passed us by :-(\n");
 	} else {
-		GGIDPRINT_MODE("display-fbdev: Change mode OK.\n");
+		DPRINT_MODE("display-fbdev: Change mode OK.\n");
 	}
-	
+
 	if (! err && (priv->fix.ypanstep == 0) && (mode->frames > 1)) {
 		mode->frames = 1;
-		GGIDPRINT_MODE("display-fbdev: cannot do vertical panning "
+		DPRINT_MODE("display-fbdev: cannot do vertical panning "
 				"-- frames reduced to 1.\n");
 	}
 
 	/* Reset panning and frames */
-        vis->d_frame_num = vis->origin_x = vis->origin_y = 0;
+	vis->d_frame_num = vis->origin_x = vis->origin_y = 0;
 
 	return err;
 }
@@ -417,7 +391,7 @@ static int do_change_mode(ggi_visual *vis, struct fb_var_screeninfo *var)
 
 static int do_mmap(ggi_visual *vis)
 {
-	ggi_fbdev_priv *priv = LIBGGI_PRIVATE(vis);
+	ggi_fbdev_priv *priv = FBDEV_PRIV(vis);
 	ggi_graphtype gt;
 	int xres_in_bytes;
 	int i;
@@ -426,12 +400,12 @@ static int do_mmap(ggi_visual *vis)
 	if (priv->fb_ptr != NULL) {
 		_GGI_free_dbs(vis);
 		/* Clear old contents */
-		clear_fbmem(priv->fb_ptr, priv->mmap_size);
-		munmap(priv->fb_ptr, priv->mmap_size);
+		clear_fbmem(priv->fb_ptr, (unsigned)priv->mmap_size);
+		munmap(priv->fb_ptr, (unsigned)priv->mmap_size);
 	}
 
         /* calculate framebuffer size */
-	gt = LIBGGI_MODE(vis)->graphtype;
+	gt = LIBGGI_GT(vis);
 
 	switch (priv->fix.type) {
 	case FB_TYPE_PLANES:
@@ -468,14 +442,15 @@ static int do_mmap(ggi_visual *vis)
 	priv->mmap_size = priv->fix.smem_len;
 #endif
 	
-	GGIDPRINT_MODE("display-fbdev: frame_size=0x%x fb_size=0x%x "
+	DPRINT_MODE("display-fbdev: frame_size=0x%x fb_size=0x%x "
 		    "mmap_size=0x%x\n", priv->frame_size,
 		    priv->fb_size, priv->mmap_size);
 
-	priv->fb_ptr = mmap(NULL, priv->mmap_size, PROT_READ | PROT_WRITE, 
+	priv->fb_ptr = mmap(NULL, (unsigned)priv->mmap_size,
+			    PROT_READ | PROT_WRITE, 
 			    MAP_SHARED, LIBGGI_FD(vis), 0);
 
-	GGIDPRINT_MODE("display-fbdev: FB_PTR=%p\n", priv->fb_ptr);
+	DPRINT_MODE("display-fbdev: FB_PTR=%p\n", priv->fb_ptr);
 
 	if (priv->fb_ptr == MAP_FAILED) {
 		priv->fb_ptr = NULL;
@@ -483,7 +458,7 @@ static int do_mmap(ggi_visual *vis)
 	}
 
 	/* clear all frames */
-	clear_fbmem(priv->fb_ptr, priv->fb_size);
+	clear_fbmem(priv->fb_ptr, (unsigned)priv->fb_size);
 
 	/* Set up pixel format */
 	memset(LIBGGI_PIXFMT(vis), 0, sizeof(ggi_pixelformat));
@@ -498,7 +473,7 @@ static int do_mmap(ggi_visual *vis)
 		break;
 
 	case GT_TRUECOLOR:
-		GGIDPRINT_MODE("fbdev: RGB %d:%d:%d offsets %d:%d:%d\n",
+		DPRINT_MODE("fbdev: RGB %d:%d:%d offsets %d:%d:%d\n",
 			priv->var.red.length, priv->var.green.length,
 			priv->var.blue.length, priv->var.red.offset,
 			priv->var.green.offset, priv->var.blue.offset);
@@ -532,7 +507,7 @@ static int do_mmap(ggi_visual *vis)
 
 		buf->frame = i;
 		buf->type  = GGI_DB_NORMAL;
-		buf->read  = (uint8 *) priv->fb_ptr + i * priv->frame_size;
+		buf->read  = (uint8_t *) priv->fb_ptr + i * priv->frame_size;
 		buf->write = buf->read;
 
 		if (priv->fix.type == FB_TYPE_PLANES) {
@@ -567,9 +542,8 @@ static int do_mmap(ggi_visual *vis)
 
 static int do_setmode(ggi_visual *vis, struct fb_var_screeninfo *var)
 {
-	ggi_fbdev_priv *priv = LIBGGI_PRIVATE(vis);
-	char libname[256], libargs[256];
-	ggi_graphtype gt;
+	ggi_fbdev_priv *priv = FBDEV_PRIV(vis);
+	char libname[GGI_MAX_APILEN], libargs[GGI_MAX_APILEN];
 	int err, id;
 
 	err = do_change_mode(vis, var);
@@ -588,9 +562,10 @@ static int do_setmode(ggi_visual *vis, struct fb_var_screeninfo *var)
 	priv->idleaccel = NULL;
 	vis->needidleaccel = 1; /* Temp hack until we work out the */
 	vis->accelactive = 0;   /* new changed() traversal for renderers */
-	for(id=1; GGI_fbdev_getapi(vis, id, libname, libargs) == 0; id++) {
-		if (_ggiOpenDL(vis, libname, libargs, NULL)) {
-			GGIDPRINT_LIBS("display-fbdev: Error opening the "
+
+	for (id=1; GGI_fbdev_getapi(vis, id, libname, libargs) == 0; id++) {
+		if (_ggiOpenDL(vis, _ggiGetConfigHandle(), libname, libargs, NULL)) {
+			DPRINT_LIBS("display-fbdev: Error opening the "
 				       "%s (%s) library.\n", libname, libargs);
 			if (id < 4) {
 				fprintf(stderr, 
@@ -603,33 +578,22 @@ static int do_setmode(ggi_visual *vis, struct fb_var_screeninfo *var)
 			priv->have_accel = 1;
 		}
 
-		GGIDPRINT_LIBS("Success in loading %s (%s)\n",
+		DPRINT_LIBS("Success in loading %s (%s)\n",
 			       libname, libargs);
 	}
 	if (!priv->have_accel) LIBGGI_GC(vis) = priv->normalgc;
 	vis->accelactive = 0;
 
-	/* Set up palette */
-	if (vis->palette) {
-		free(vis->palette);
-		vis->palette = NULL;
-	}
-	gt = LIBGGI_GT(vis);
-	if ((GT_SCHEME(gt) == GT_PALETTE) || (GT_SCHEME(gt) == GT_TEXT)) {
-	    	int nocols = 1 << GT_DEPTH(gt);
+	GGI_fbdev_color_setup(vis);
+	if (vis->opcolor->setpalvec) ggiSetColorfulPalette(vis);
+	if (vis->opcolor->setgammamap) ggiSetGamma(vis, 1.0, 1.0, 1.0);
 
-		vis->palette = _ggi_malloc(nocols * sizeof(ggi_color));
-		vis->opcolor->setpalvec = GGI_fbdev_setpalvec;
-		/* Initialize palette */
-		ggiSetColorfulPalette(vis);
-	}
-	
 	vis->opdraw->setorigin = GGI_fbdev_setorigin;
 	vis->opdraw->setdisplayframe = GGI_fbdev_setdisplayframe;
 
 	ggiIndicateChange(vis, GGI_CHG_APILIST);
 
-	GGIDPRINT_MODE("display-fbdev: do_setmode SUCCESS\n");
+	DPRINT_MODE("display-fbdev: do_setmode SUCCESS\n");
 
 	return 0;
 }
@@ -638,12 +602,21 @@ static int do_setmode(ggi_visual *vis, struct fb_var_screeninfo *var)
 int GGI_fbdev_setmode(ggi_visual *vis, ggi_mode *mode)
 {
 	struct fb_var_screeninfo var;
+	struct ggi_fbdev_timing *timing;
 	int err;
+	ggi_fbdev_priv *priv = FBDEV_PRIV(vis);
 
-	err = do_checkmode(vis, mode, &var);
+	err = _GGI_fbdev_do_checkmode( vis, mode, &timing );
         if (err != 0) return err;
 
-	GGIDPRINT_MODE("display-fbdev: setmode %dx%d#%dx%dF%d[0x%02x]\n",
+	/* Construct var from timing and mode and priv->orig_var */
+	var = priv->orig_var;
+	var.nonstd = 0;
+	var.xoffset = var.yoffset = 0;
+	ggimode2var(vis, mode, &var);
+	timing2var(timing, &var, mode->frames );
+
+	DPRINT_MODE("display-fbdev: setmode %dx%d#%dx%dF%d[0x%02x]\n",
 			mode->visible.x, mode->visible.y,
 			mode->virt.x, mode->virt.y, 
 			mode->frames, mode->graphtype);
@@ -654,21 +627,21 @@ int GGI_fbdev_setmode(ggi_visual *vis, ggi_mode *mode)
 	err = do_setmode(vis, &var);
 	if (err != 0) return err;
 
-	GGIDPRINT_MODE("display-fbdev: setmode success.\n");
+	DPRINT_MODE("display-fbdev: setmode success.\n");
 
 	return 0;
 }
 
 
-int GGI_fbdev_resetmode(ggi_visual *vis)
+int GGI_fbdev_mode_reset(ggi_visual *vis)
 {
-	ggi_fbdev_priv *priv = LIBGGI_PRIVATE(vis);
+	ggi_fbdev_priv *priv = FBDEV_PRIV(vis);
 
 	if (priv->fb_ptr != NULL) {
 		_GGI_free_dbs(vis);
 		/* Clear framebuffer */
-		clear_fbmem(priv->fb_ptr, priv->mmap_size);
-		munmap(priv->fb_ptr, priv->mmap_size);
+		clear_fbmem(priv->fb_ptr, (unsigned)priv->mmap_size);
+		munmap(priv->fb_ptr, (unsigned)priv->mmap_size);
 	}
 	fbdev_doioctl(vis, FBIOPUT_VSCREENINFO, &priv->orig_var);
 	if (priv->fix.xpanstep != 0 || priv->fix.ypanstep != 0) {
@@ -678,17 +651,58 @@ int GGI_fbdev_resetmode(ggi_visual *vis)
 	return 0;
 }
 
-
-static int
-do_checkmode(ggi_visual *vis, ggi_mode *mode, struct fb_var_screeninfo *var)
+/* Convert the fbdev specific ggi_fbdev_timing structure
+ * into a generic ggi_mode structure suitable for 
+ * passing as suggested mode to _GGI_generic_mode_update()
+ */
+static
+void _GGI_fbdev_checkmode_adapt( ggi_mode * m, 
+				struct ggi_fbdev_timing * timing,
+				ggi_fbdev_priv * priv ) 
 {
-	ggi_fbdev_priv *priv = LIBGGI_PRIVATE(vis);
-	ggi_graphtype gt = mode->graphtype;
-	ggi_mode oldmode = *mode;
+	m->visible.x = timing->xres;
+	m->visible.y = timing->yres;
+	m->virt.x = timing->xres_virtual;
+	m->virt.y = timing->yres_virtual;
+
+	/* In the future we may want to support smaller than fullscreen
+	 * visuals to emulate unsupported modes.  If so, we'll need to
+	 * handle size in the adjust function since it depends on the 
+	 * requested resolution... Something like this:
+	_ggi_physz_figure_size(sug, priv->physzflags, 
+				     &(priv->physz),
+				(signed)(DPI(width, x)),
+				(signed)(DPI(height,y)), 
+				sug->visible.x, sug->visible.y);
+	*/
+	/* For now, let's just use the framebuffer's values for the
+	 * size of the whole screen...
+	 * Note: the cast is neccessary because orig_var.width has an
+	 * unsigned type! */
+	m->size.x = ((signed)priv->orig_var.width <= 0 ) 
+		? GGI_AUTO : priv->orig_var.width;
+	m->size.y = ((signed)priv->orig_var.height <= 0 ) 
+		?  GGI_AUTO : priv->orig_var.height;
+
+	/* handled by the adjust function: 
+	 * frames, graphtype, dpp 
+	 */
+}
+
+/* Adjust suggested mode sug to match against
+ * requested mode req for generic modecheck. 
+ */
+static
+void _GGI_fbdev_checkmode_adjust( ggi_mode *req,
+				  ggi_mode *sug,
+				  ggi_fbdev_priv *priv )
+{
+
+	ggi_graphtype gt = req->graphtype;
 	int xdpp, ydpp;
 	int maxframes;
-	int err = 0, ret;
 
+	/* graphtype... */
 	/* Handle GGI_AUTO first */
 	if (gt == GT_AUTO) {
 #ifdef FB_TYPE_TEXT
@@ -735,116 +749,159 @@ do_checkmode(ggi_visual *vis, ggi_mode *mode, struct fb_var_screeninfo *var)
 		}
 	}
 
-	mode->graphtype = _GGIhandle_gtauto(gt);
+	sug->graphtype = _GGIhandle_gtauto(gt);
 
-	xdpp = (GT_SCHEME(mode->graphtype) == GT_TEXT) ? FB_KLUDGE_FONTX : 1;
-	ydpp = (GT_SCHEME(mode->graphtype) == GT_TEXT) ? FB_KLUDGE_FONTY : 1;
-	if ((mode->dpp.x != xdpp && mode->dpp.x != GGI_AUTO) ||
-	    (mode->dpp.y != ydpp && mode->dpp.y != GGI_AUTO)) {
-		err = -1;
+	/* dpp... */
+	xdpp = (GT_SCHEME(sug->graphtype) == GT_TEXT) ? FB_KLUDGE_FONTX : 1;
+	ydpp = (GT_SCHEME(sug->graphtype) == GT_TEXT) ? FB_KLUDGE_FONTY : 1;
+
+	sug->dpp.x = xdpp;
+	sug->dpp.y = ydpp;
+
+	/* frames... */
+	sug->frames = req->frames;
+	if (sug->frames == GGI_AUTO) {
+		sug->frames = 1;
+	} else if (priv->orig_fix.ypanstep == 0 && sug->frames > 1) {
+		DPRINT_MODE("display-fbdev: "
+			    "%d frames but no vertical panning\n", sug->frames);
+		sug->frames = 1;
 	}
-	mode->dpp.x = xdpp;
-	mode->dpp.y = ydpp;
-
-	if (mode->frames == GGI_AUTO) {
-		mode->frames = 1;
-	} else if (priv->orig_fix.ypanstep == 0 && mode->frames > 1) {
-		GGIDPRINT_MODE("display-fbdev: %d frames but no vertical panning\n", mode->frames);
-		mode->frames = 1;
-		err = -1;
-	}
-
-	*var = priv->orig_var;
-
-	var->nonstd = 0;
-	var->xoffset = var->yoffset = 0;
-
-	ggimode2var(vis, mode, var);
-
-	/* Try to get the timing from the standard database */
-	ret = get_timing(vis, var, mode->frames);
-	if (ret < 0) {
-		GGIDPRINT_MODE("display-fbdev: unable to find timing for mode\n");
-		if (priv->need_timings &&
-		    (var->xres != priv->orig_var.xres ||
-		     var->yres != priv->orig_var.yres)) {
-			var2ggimode(vis, &priv->orig_var, mode, mode->frames);
-			return -1;
-		}
-	} else if (ret != 0) {
-		err = -1;
-	}
-
-	/* Just check the mode */
-	var->activate = FB_ACTIVATE_TEST;
-
-	/* Check mode with framebuffer */
-	if (fbdev_doioctl(vis, FBIOPUT_VSCREENINFO, var) < 0) {
-		GGIDPRINT_MODE("display-fbdev: FB_ACTIVATE_TEST failed\n");
-		err = -1;
-	}
-
-	var2ggimode(vis, var, mode, mode->frames);
-
-	if ((oldmode.visible.x && mode->visible.x != oldmode.visible.x) ||
-	    (oldmode.visible.y && mode->visible.y != oldmode.visible.y) ||
-	    (oldmode.virt.x && mode->virt.x != oldmode.virt.x) ||
-	    (oldmode.virt.y && mode->virt.y != oldmode.virt.y) ||
-	    (GT_DEPTH(oldmode.graphtype) &&
-	     GT_DEPTH(mode->graphtype) != GT_DEPTH(oldmode.graphtype)) ||
-	    (GT_SIZE(oldmode.graphtype) &&
-	     GT_SIZE(mode->graphtype) != GT_SIZE(oldmode.graphtype)) ||
-	    (GT_SCHEME(oldmode.graphtype) &&
-	     GT_SCHEME(mode->graphtype) != GT_SCHEME(oldmode.graphtype))) {
-		GGIDPRINT_MODE("display-fbdev: checkmode error\n");
-		err = -1;
-	}
-
-#define DPI(dim, d) \
-	((priv->orig_var.dim <= 0) ? 0 : \
-	 (mode->visible.d * mode->dpp.d * 254 / priv->orig_var.dim / 10))
-
-	if (!err) {
-		err = _ggi_figure_physz(mode, priv->physzflags, &(priv->physz),
-					DPI(width, x), DPI(height,y), 
-					mode->visible.x, mode->visible.y);
-	}
-
-#undef DPI
 
 	maxframes = priv->orig_fix.smem_len /
-		(mode->virt.x * mode->virt.y *
-		 GT_ByPP(mode->graphtype));
-	if (mode->frames > maxframes) {
-		mode->frames = maxframes;
-		err = -1;
+		(sug->virt.x * sug->virt.y *
+		 GT_ByPP(sug->graphtype));
+	if (sug->frames > maxframes) {
+		sug->frames = maxframes;
 	}
 
-	GGIDPRINT_MODE("display-fbdev: checkmode returns %dx%d#%dx%dF%d[0x%02x]\n",
-			mode->visible.x, mode->visible.y,
-			mode->virt.x, mode->virt.y, 
-			mode->frames, mode->graphtype);
 
-	return err;
 }
 
 
-int GGI_fbdev_checkmode(ggi_visual *vis, ggi_mode *mode)
+/* Iterate over the timings list obtained from the fb.modes 
+ * file and search for the best matching mode. 
+ */
+static 
+int _GGI_fbdev_do_checkmode(ggi_visual *vis, ggi_mode *mode,
+                            struct ggi_fbdev_timing **timing_)
 {
-	struct fb_var_screeninfo var;
+	int err = GGI_OK;
+	ggi_fbdev_priv *priv = FBDEV_PRIV(vis);
+	ggi_checkmode_t * cm;
 
-	GGIDPRINT_MODE("display-fbdev: checkmode %dx%d#%dx%dF%d[0x%02x]\n",
+	struct ggi_fbdev_timing *timing; /* iteration variable */
+
+	/* Keep track of the previous node while we iterate in order to 
+	 * delete bad modes from our list. */
+	struct ggi_fbdev_timing *prev = NULL;
+	struct ggi_fbdev_timing *saved_timing;
+
+
+	cm = _GGI_generic_checkmode_create();
+	
+
+	DPRINT_MODE("display-fbdev: checkmode %dx%d#%dx%dF%d[0x%02x]\n",
 			mode->visible.x, mode->visible.y,
 			mode->virt.x, mode->virt.y, 
 			mode->frames, mode->graphtype);
 
-	return do_checkmode(vis, mode, &var);
+
+	/* This puts the contents of mode into cm->req */
+	_GGI_generic_checkmode_init( cm, mode );
+
+	/* The first timing will be saved in case all nodes are deleted.
+	 * We will use this as a fallback mode suggestion since it was 
+	 * ostensibly obtained by querying the framebuffer for the current 
+	 * mode. */
+	saved_timing = priv->timings;
+
+	timing = priv->timings;
+	while( timing )  {
+		struct fb_var_screeninfo var;
+
+		/* Convert timing structure to a suggested mode */
+		_GGI_fbdev_checkmode_adapt( mode, timing, priv );
+
+		/* Use the requested mode to tailor the suggestion */
+		_GGI_fbdev_checkmode_adjust( &cm->req, mode, priv );
+
+
+		/* construct structure for passing to ioctl */
+		var =  priv->orig_var;
+		var.nonstd = 0;
+		var.xoffset = var.yoffset = 0;
+		ggimode2var( vis, mode, &var );
+		timing2var( timing, &var, mode->frames );
+
+
+		/* Check mode with framebuffer */
+		var.activate = FB_ACTIVATE_TEST; /* just check */
+		if (fbdev_doioctl(vis, FBIOPUT_VSCREENINFO, &var) >= 0) {
+			/* The framebuffer seems to like it, so suggest 
+			 * it as a valid mode to the checkmode API. */
+			var2ggimode(vis, &var, mode, mode->frames);
+			_GGI_generic_checkmode_update( cm, mode, (intptr_t)timing );
+		}
+		else {
+			/* The framebuffer didn't like this mode, so
+			 * let's delete it from the timings list... */
+			struct ggi_fbdev_timing *next;
+			next = timing->next;
+			if( timing == priv->timings ) 
+				priv->timings = next;
+			else
+				prev->next = next;
+			/* if it's the saved_timing, it will be freed
+			 * later after we are certain we haven't freed
+			 * the entire list... */
+			if( timing != saved_timing )
+				free( timing );
+			timing = next;
+			continue;
+		}
+		prev = timing;
+		timing = timing->next;
+	}
+
+	if( priv->timings == NULL )
+	{
+		/* No good modes.. perhaps the docs for ggiCheckMode should
+		 * specify a return value for when this seems to be the case.
+		 * Let's suggest the saved_timing... */
+		DPRINT_MODE( "Error! FB_ACTIVATE_TEST failed for all modes\n" );
+
+		_GGI_fbdev_checkmode_adapt( mode, saved_timing, priv );
+		_GGI_fbdev_checkmode_adjust( &cm->req, mode, priv );
+		_GGI_generic_checkmode_update( cm, mode, 
+					       (intptr_t)saved_timing );
+
+		/* ...and restore it to the list. */
+		saved_timing->next = NULL;
+		priv->timings = saved_timing;
+
+		/* QUESTION: If the saved_timing mode just happens to
+		 * match the reqested mode, _GGI_generic_checkmode_finish
+		 * will return success.  Is this the proper behavior? */
+	}
+	else if( saved_timing != priv->timings )
+		free( saved_timing );
+
+	err = _GGI_generic_checkmode_finish( cm, mode, (intptr_t *)timing_);
+
+	_GGI_generic_checkmode_destroy( cm );
+	return err;
+}
+
+int GGI_fbdev_checkmode(ggi_visual *vis, ggi_mode *mode)
+{
+	return _GGI_fbdev_do_checkmode( vis, mode, NULL );
 }
 
 
 int GGI_fbdev_getmode(ggi_visual *vis, ggi_mode *mode)
 {
-	GGIDPRINT_MODE("display-fbdev: getmode\n");
+	DPRINT_MODE("display-fbdev: getmode\n");
 	
 	memcpy(mode, LIBGGI_MODE(vis), sizeof(ggi_mode));
 
