@@ -1,4 +1,4 @@
-/* $Id: mode.c,v 1.3 2002/09/29 19:27:42 skids Exp $
+/* $Id: mode.c,v 1.13 2005/09/19 18:46:42 cegger Exp $
 ******************************************************************************
 
    LibGGI GLIDE target - Mode management.
@@ -36,6 +36,11 @@
 #include <ggi/display/glide.h>
 #include "../../default/color/color.h"
 
+#include "../common/pixfmt-setup.inc"
+#include "../common/ggi-auto.inc"
+#include "../common/gt-auto.inc"
+
+
 static void
 GGI_glide_gcchanged(ggi_visual *vis, int mask)
 {
@@ -89,7 +94,7 @@ GGI_glide_gcchanged(ggi_visual *vis, int mask)
 }
 
 
-static int glide_acquire(ggi_resource *res, uint32 actype)
+static int glide_acquire(ggi_resource *res, uint32_t actype)
 {
 	ggi_directbuffer *dbuf;
 	ggi_visual *vis;
@@ -98,7 +103,7 @@ static int glide_acquire(ggi_resource *res, uint32 actype)
 	GrBuffer_t buffer;
 	GrLfbInfo_t inf;
 
-	GGIDPRINT_MISC("glide_acquire(%p, 0x%x) called\n", res, actype);
+	DPRINT_MISC("glide_acquire(%p, 0x%x) called\n", res, actype);
 
 	if (actype & ~(GGI_ACTYPE_READ | GGI_ACTYPE_WRITE)) {
 		return GGI_EARGINVAL;
@@ -161,7 +166,7 @@ static int glide_acquire(ggi_resource *res, uint32 actype)
 	res->curactype |= actype;
 	res->count++;
 
-	GGIDPRINT_MISC("glide_acquire - success, count: %d\n", res->count);
+	DPRINT_MISC("glide_acquire - success, count: %d\n", res->count);
 
 	return 0;
 }
@@ -174,7 +179,7 @@ static int glide_release(ggi_resource *res)
 	int bufnum;
 	GrBuffer_t buffer;
 
-	GGIDPRINT_MISC("glide_release(%p) called\n", res);
+	DPRINT_MISC("glide_release(%p) called\n", res);
 
 	if (res->count < 1) return GGI_ENOTALLOC;
 
@@ -219,23 +224,20 @@ int GGI_glide_flush(ggi_visual *vis, int x, int y, int w, int h, int tryflag)
 
 int GGI_glide_getapi(ggi_visual *vis, int num, char *apiname, char *arguments)
 {
-	switch(num)
-	{
+	*arguments = '\0';
+	switch(num) {
 	case 0:
 		strcpy(apiname, "display-glide");
-		strcpy(arguments, "");
 		return 0;
 	case 1:
 		strcpy(apiname, "generic-stubs");
-		strcpy(arguments, "");
 		return 0;
 	case 2:
 		strcpy(apiname, "generic-color");
-		strcpy(arguments, "");
 		return 0;
 	}
 	
-	return -1;
+	return GGI_ENOMATCH;
 }
 
 /* The minimum frequency glide can do */
@@ -274,7 +276,7 @@ getglidefreq(glide_priv *priv, ggi_mode *mode)
 }
 
 static const struct {
-	sint16	x, y;
+	int16_t	x, y;
 	GrScreenResolution_t glideres;
 } glide_resolutions[] = {
 #if 0 /* As of Glide 2.51 these modes are not supported */
@@ -315,12 +317,12 @@ res2glideres(glide_priv *priv, ggi_mode *mode,
 	for (i = 0; glide_resolutions[i].x != 0; i++) {
 		if (glide_resolutions[i].x * glide_resolutions[i].y * 4
 		    > priv->fbmem) {
-			GGIDPRINT_MODE("res2glideres: breaking on nomem\n");
+			DPRINT_MODE("res2glideres: breaking on nomem\n");
 			break;
 		}
 		if (glide_resolutions[i].y * GGIGLIDE_MINFREQ
 		    > priv->maxhfreq * 1000) {
-			GGIDPRINT_MODE("res2glideres: breaking on highfreq\n");
+			DPRINT_MODE("res2glideres: breaking on highfreq\n");
 			break;
 		}
 			
@@ -352,18 +354,20 @@ res2glideres(glide_priv *priv, ggi_mode *mode,
 		
 int GGI_glide_setmode(ggi_visual *vis, ggi_mode *mode)
 {
-	glide_priv *priv = LIBGGI_PRIVATE(vis);
-	char libname[256], libargs[256];
+	int rc = GGI_OK;
+	glide_priv *priv = GLIDE_PRIV(vis);
+	char libname[GGI_MAX_APILEN], libargs[GGI_MAX_APILEN];
 	GrScreenResolution_t resolution;
 	GrScreenRefresh_t    freq;
 	GrLfbInfo_t inf;
 	int id;
 	int i;
 
-	if (GGI_glide_checkmode(vis, mode)) return -1;
+	rc = GGI_glide_checkmode(vis, mode);
+	if (rc < 0) return rc;
 
 	if (res2glideres(priv, mode, &resolution) != 0) {
-		LIBGGI_ASSERT(0, "Invalid Glide resolution!");
+		LIB_ASSERT(0, "Invalid Glide resolution!");
 	}
 	freq = getglidefreq(priv, mode);
 
@@ -371,61 +375,44 @@ int GGI_glide_setmode(ggi_visual *vis, ggi_mode *mode)
 		grSstWinClose();
 	}
 	
-	GGIDPRINT_MODE("resolution: 0x%02x, freq: 0x%02x\n", resolution, freq);
+	DPRINT_MODE("resolution: 0x%02x, freq: 0x%02x\n", resolution, freq);
 	if (grSstWinOpen(0, resolution, freq,
 			 GR_COLORFORMAT_ABGR, GR_ORIGIN_UPPER_LEFT, 2, 0)
 	    != FXTRUE) {
-		GGIDPRINT_MODE("FAIL!!!");
-		return -1;
+		DPRINT_MODE("FAIL!!!");
+		return GGI_ENOMATCH;
 	}
 
 	/* Fill in ggi_pixelformat */
 	memset(LIBGGI_PIXFMT(vis), 0, sizeof(ggi_pixelformat));
-	LIBGGI_PIXFMT(vis)->depth=GT_DEPTH(mode->graphtype);
-	LIBGGI_PIXFMT(vis)->size=GT_SIZE(mode->graphtype);
+	setup_pixfmt(LIBGGI_PIXFMT(vis), mode->graphtype);
 
 	switch(mode->graphtype)	{
 	case GT_15BIT:
-		LIBGGI_PIXFMT(vis)->blue_mask  = ((1<<5)-1);
-		LIBGGI_PIXFMT(vis)->green_mask = ((1<<5)-1)<<5;
-		LIBGGI_PIXFMT(vis)->red_mask   = ((1<<5)-1)<<(5+5);
-
 		priv->src_format = GR_LFB_SRC_FMT_555;
 		priv->write_mode = GR_LFBWRITEMODE_555;
 		priv->bytes_per_pixel = 2;
 		break;
 		
 	case GT_16BIT:
-		LIBGGI_PIXFMT(vis)->blue_mask  = ((1<<5)-1);
-		LIBGGI_PIXFMT(vis)->green_mask = ((1<<6)-1)<<5;
-		LIBGGI_PIXFMT(vis)->red_mask   = ((1<<5)-1)<<(5+6);
-
 		priv->src_format = GR_LFB_SRC_FMT_565;
 		priv->write_mode = GR_LFBWRITEMODE_565;
 		priv->bytes_per_pixel = 2;
 		break;
 		
 	case GT_24BIT:
-		LIBGGI_PIXFMT(vis)->blue_mask  = ((1<<8)-1);
-		LIBGGI_PIXFMT(vis)->green_mask = ((1<<8)-1)<<8;
-		LIBGGI_PIXFMT(vis)->red_mask   = ((1<<8)-1)<<(8+8);
-
 		priv->src_format = GR_LFB_SRC_FMT_888;
 		priv->write_mode = GR_LFBWRITEMODE_888;
 		priv->bytes_per_pixel = 3;
 		break;
 		
 	case GT_32BIT:
-		LIBGGI_PIXFMT(vis)->blue_mask  = ((1<<8)-1);
-		LIBGGI_PIXFMT(vis)->green_mask = ((1<<8)-1)<<8;
-		LIBGGI_PIXFMT(vis)->red_mask   = ((1<<8)-1)<<(8+8);
-
 		priv->src_format = GR_LFB_SRC_FMT_8888;
 		priv->write_mode = GR_LFBWRITEMODE_8888;
 		priv->bytes_per_pixel = 4;
 		break;
 	default:
-		LIBGGI_ASSERT(0, "Glide: Illegal mode!");
+		LIB_ASSERT(0, "Glide: Illegal mode!");
 	}
 	_ggi_build_pixfmt(LIBGGI_PIXFMT(vis));
 
@@ -476,7 +463,7 @@ int GGI_glide_setmode(ggi_visual *vis, ggi_mode *mode)
 				= inf.strideInBytes;
 			LIBGGI_APPBUFS(vis)[i]->buffer.plb.pixelformat
 				= LIBGGI_PIXFMT(vis);
-			GGIDPRINT_MODE("DB: %d, addr: %p, stride: %d\n", i, 
+			DPRINT_MODE("DB: %d, addr: %p, stride: %d\n", i, 
 				       LIBGGI_APPBUFS(vis)[i]->read,
 				       LIBGGI_APPBUFS(vis)[i]->buffer.plb.stride);
 		}
@@ -492,13 +479,13 @@ int GGI_glide_setmode(ggi_visual *vis, ggi_mode *mode)
 	for (id = 1; GGI_glide_getapi(vis, id, libname, libargs) == 0; id++) {
 		int err;
 
-		err = _ggiOpenDL(vis, libname, libargs, NULL);
+		err = _ggiOpenDL(vis, _ggiGetConfigHandle(), libname, libargs, NULL);
 		if (err) {
 			fprintf(stderr, "display-glide: Error opening the "
 				"%s (%s) library\n", libname, libargs);
 			return err;
 		}
-		GGIDPRINT ("Success in loading %s (%s)\n", libname, libargs);
+		DPRINT ("Success in loading %s (%s)\n", libname, libargs);
 	}
 	
 	/* Drawing functions */
@@ -534,12 +521,17 @@ int GGI_glide_setmode(ggi_visual *vis, ggi_mode *mode)
 	vis->opgc->gcchanged = GGI_glide_gcchanged;
 
 	/* Text */
-	if (GT_SIZE(mode->graphtype) == 16) {
+	switch (GT_SIZE(mode->graphtype)) {
+	case 16:
 		vis->opdraw->putc = GGI_glide16_putc;
-	} else if (GT_SIZE(mode->graphtype) == 32) {
+		break;
+	case 32:
 		vis->opdraw->putc = GGI_glide32_putc;
+		break;
+	default:
+		break;
 	}
-	
+
 	ggiIndicateChange(vis, GGI_CHG_APILIST);
 
 	priv->setmodesuccess = 1;
@@ -552,126 +544,48 @@ int GGI_glide_checkmode(ggi_visual *vis, ggi_mode *tm)
 	GrScreenResolution_t resolution; /* Dummy */
 	int err = 0;
 
-	LIBGGI_APPASSERT(vis != NULL, "glide: Visual NULL in GGIcheckmode");
-	
+	APP_ASSERT(vis != NULL, "glide: Visual NULL in GGIcheckmode");
+
+	/* handle AUTO */
+	_GGIhandle_ggiauto(mode, 640, 480);
+
+	mode->graphtype = _GGIhandle_gtauto(mode->graphtype);
+
 	if (tm->frames == GGI_AUTO) {
 		tm->frames = 1;
 	} else if (tm->frames < 1) {
-		err = -1;
+		err = GGI_ENOMATCH;
 		tm->frames = 1;
 	} else if (tm->frames > 2) {
-		err = -1;
+		err = GGI_ENOMATCH;
 		tm->frames = 2;
 	}
 
-	if (tm->visible.x == GGI_AUTO && tm->visible.y == GGI_AUTO) {
-		tm->visible.x = 640;
-		tm->visible.y = 480;
-	} else if (res2glideres(GLIDE_PRIV(vis), tm, &resolution) != 0) {
-		err = -1;
+	if (res2glideres(GLIDE_PRIV(vis), tm, &resolution) != 0) {
+		err = GGI_ENOMATCH;
 	}
 
-	if (tm->virt.x == GGI_AUTO) {
-		tm->virt.x = tm->visible.x;
-	}
-	if (tm->virt.y == GGI_AUTO) {
-		tm->virt.y = tm->visible.y;
-	}	
-
 	if (tm->virt.x < tm->visible.x) {
 		tm->virt.x = tm->visible.x;
-		err = -1;
+		err = GGI_ENOMATCH;
 	}
 	if (tm->virt.y < tm->visible.y) {
 		tm->virt.y = tm->visible.y;
-		err = -1;
+		err = GGI_ENOMATCH;
 	}
 
 	if ((tm->dpp.x != 1 && tm->dpp.x != GGI_AUTO) ||
 	    (tm->dpp.y != 1 && tm->dpp.y != GGI_AUTO)) {
-		err = -1;
+		err = GGI_ENOMATCH;
 	}
 	tm->dpp.x = tm->dpp.y = 1;
-
-	if (tm->size.x != GGI_AUTO || tm->size.y != GGI_AUTO) {
-		err = -1;
-	}
-	tm->size.x = tm->size.y = GGI_AUTO;
-
-	if (GT_SCHEME(tm->graphtype) != GT_TRUECOLOR
-	    && GT_SCHEME(tm->graphtype) != GT_AUTO) {
-		err = -1;
-	}
-	GT_SETSCHEME(tm->graphtype, GT_TRUECOLOR);
-	if (GT_SUBSCHEME(tm->graphtype) != 0
-	    && GT_SUBSCHEME(tm->graphtype) != GT_AUTO) {
-		err = -1;
-	}
-	GT_SETSUBSCHEME(tm->graphtype, 0);
-
-	/* FIXME!
-	   Could someone please come up with a nice way to handle
-	   SIZE/DEPTH properly...
-	*/
-	switch (GT_DEPTH(tm->graphtype)) {
-	case 15:
-	case 16:
-		if (GT_SIZE(tm->graphtype) != 16
-		    && GT_SIZE(tm->graphtype) != GT_AUTO) {
-			err = -1;
-		}
-		GT_SETSIZE(tm->graphtype, 16);
-		break;
-	case 24:
-		if (GT_SIZE(tm->graphtype) != 24
-		    && GT_SIZE(tm->graphtype) != 32
-		    && GT_SIZE(tm->graphtype) != GT_AUTO) {
-			err = -1;
-			GT_SETSIZE(tm->graphtype, 32);
-		} else if (GT_SIZE(tm->graphtype) == GT_AUTO) {
-			GT_SETSIZE(tm->graphtype, 32);
-		}
-		break;
-	case GT_AUTO:
-		break;
-	default:
-		GT_SETDEPTH(tm->graphtype, GT_AUTO);
-		err = -1;
-		break;
-	}
-	switch (GT_SIZE(tm->graphtype)) {
-	case 16:
-		if (GT_DEPTH(tm->graphtype) != 15
-		    && GT_DEPTH(tm->graphtype) != 16
-		    && GT_DEPTH(tm->graphtype) != GT_AUTO) {
-			err = -1;
-			GT_SETDEPTH(tm->graphtype, 16);
-		} else if (GT_DEPTH(tm->graphtype) == GT_AUTO) {
-			GT_SETDEPTH(tm->graphtype, 16);
-		}
-		break;
-	case 24:
-	case 32:
-		if (GT_DEPTH(tm->graphtype) != 24
-		    && GT_DEPTH(tm->graphtype) != GT_AUTO) {
-			err = -1;
-		}
-		GT_SETDEPTH(tm->graphtype, 24);
-		break;
-	case GT_AUTO:
-		break;
-	default:
-		GT_SETSIZE(tm->graphtype, GT_AUTO);
-		err = -1;
-		break;
-	}
-	if (GT_DEPTH(tm->graphtype) == GT_AUTO) {
-		tm->graphtype = GT_16BIT;
-	} else if (GT_SIZE(tm->graphtype) == GT_AUTO) {
-		GT_SETSIZE(tm->graphtype, (GT_DEPTH(tm->graphtype)+1)&~8);
-	}
-
-        return err;
+	if (err) return err;
+#if 0
+	err = _ggi_physz_figure_size(mode, GLIDE_PRIV(vis)->physzflags,
+				&(GLIDE_PRIV(vis)->physz),
+				0, 0, mode->visible.x, mode->visible.y);
+#endif
+	return err;
 }
 
 /*
@@ -679,7 +593,8 @@ int GGI_glide_checkmode(ggi_visual *vis, ggi_mode *tm)
 */
 int GGI_glide_getmode(ggi_visual *vis, ggi_mode *tm)
 {
-	LIBGGI_APPASSERT(vis != NULL, "glide: GGIgetmode: Visual == NULL");
+	APP_ASSERT(vis != NULL,
+		"display-glide: GGIgetmode: Visual == NULL");
 	
 	/* We assume the mode in the visual to be OK 
 	*/
