@@ -1,4 +1,4 @@
-/* $Id: visual.c,v 1.3 2002/12/04 23:06:02 cegger Exp $
+/* $Id: visual.c,v 1.11 2005/09/19 18:46:42 cegger Exp $
 ******************************************************************************
 
    GLIDE target - Initialization
@@ -32,6 +32,7 @@
 #include "config.h"
 #include <ggi/display/glide.h>
 #include <ggi/display/linvtsw.h>
+#include <ggi/internal/gg_replace.h>
 
 
 void _GGI_glide_freedbs(ggi_visual *vis)
@@ -74,7 +75,7 @@ switchback(void *vis)
 
 
 static int GGIopen(ggi_visual *vis, struct ggi_dlhandle *dlh,
-			const char *args, void *argptr, uint32 *dlret)
+			const char *args, void *argptr, uint32_t *dlret)
 {
 	ggi_linvtsw_arg vtswarg;
 	GrHwConfiguration hwconf;
@@ -88,7 +89,7 @@ static int GGIopen(ggi_visual *vis, struct ggi_dlhandle *dlh,
 	gii_input *inp;
 	int err;
 	
-	GGIDPRINT("GLIDE-lib starting\n");
+	DPRINT("GLIDE-lib starting\n");
 
 	if (args != NULL) {
 		currentcard = strtol(args, NULL, 0);
@@ -147,7 +148,8 @@ static int GGIopen(ggi_visual *vis, struct ggi_dlhandle *dlh,
 		}
 		vtswarg.novt = novt;
 
-		err = _ggiAddDL(vis, "helper-linux-vtswitch", NULL,
+		err = _ggiAddDL(vis, _ggiGetConfigHandle(),
+				"helper-linux-vtswitch", NULL,
 				&vtswarg, 0);
 		if (err) {
 			vtnum = -1;
@@ -161,7 +163,7 @@ static int GGIopen(ggi_visual *vis, struct ggi_dlhandle *dlh,
 		}
 
 		if (vtnum != -1) {
-			sprintf(strbuf, "linux-kbd:/dev/tty%d", vtnum);
+			snprintf(strbuf, 32, "linux-kbd:/dev/tty%d", vtnum);
 			inputname = strbuf;
 		}
 		on_linux_cons = 1;
@@ -174,7 +176,7 @@ static int GGIopen(ggi_visual *vis, struct ggi_dlhandle *dlh,
 		vis->input = giiOpen(inputname, NULL);
 		if (vis->input == NULL) {
 			if (vtnum != -1) {
-				sprintf(strbuf, "linux-kbd:/dev/vc/%d", vtnum);
+				snprintf(strbuf, 32, "linux-kbd:/dev/vc/%d", vtnum);
 				vis->input = giiOpen(inputname, NULL);
 			}
 			if (vis->input == NULL) {
@@ -217,7 +219,7 @@ static int GGIopen(ggi_visual *vis, struct ggi_dlhandle *dlh,
 		= priv->hwconf.SSTs[currentcard].sstBoard.VoodooConfig.fbRam
 		* 1024 * 1024;
 	priv->setmodesuccess = 0;
-	LIBGGI_PRIVATE(vis) = priv;
+	MONOTEXT_PRIV(vis) = priv;
 
 	/* Has mode management */
 	vis->opdisplay->flush = GGI_glide_flush;
@@ -233,7 +235,7 @@ static int GGIopen(ggi_visual *vis, struct ggi_dlhandle *dlh,
 
 static int GGIclose(ggi_visual *vis, struct ggi_dlhandle *dlh)
 {
-	glide_priv *priv = LIBGGI_PRIVATE(vis);
+	glide_priv *priv = MONOTEXT_PRIV(vis);
 
 	giiClose(vis->input);
 	vis->input = NULL;
@@ -247,17 +249,20 @@ static int GGIclose(ggi_visual *vis, struct ggi_dlhandle *dlh)
 }
 
 
+EXPORTFUNC
+int GGIdl_glide(int func, void **funcptr);
+
 int GGIdl_glide(int func, void **funcptr)
 {
 	switch (func) {
 	case GGIFUNC_open:
-		*funcptr = GGIopen;
+		*funcptr = (void *)GGIopen;
 		return 0;
 	case GGIFUNC_exit:
 		*funcptr = NULL;
 		return 0;
 	case GGIFUNC_close:
-		*funcptr = GGIclose;
+		*funcptr = (void *)GGIclose;
 		return 0;
 	default:
 		*funcptr = NULL;
