@@ -1,4 +1,4 @@
-/* $Id: text.c,v 1.1.1.1 2001/05/12 23:02:15 cegger Exp $
+/* $Id: text.c,v 1.3 2004/09/12 20:13:40 cegger Exp $
 ******************************************************************************
 
    Display-monotext: displaying text
@@ -25,17 +25,18 @@
 ******************************************************************************
 */
 
+#include "config.h"
+#include <ggi/display/monotext.h>
+
 #include <stdio.h>
 #include <stdlib.h>
 
-#include <ggi/display/monotext.h>
-
 #undef putc
 
 
 int GGI_monotext_putc(ggi_visual *vis, int x, int y, char c)
 {
-	ggi_monotext_priv *priv = LIBGGI_PRIVATE(vis);
+	ggi_monotext_priv *priv = MONOTEXT_PRIV(vis);
 
 	int char_w, char_h;
 	int err;
