@@ -1,4 +1,4 @@
-/* $Id: palemu.c,v 1.3 2002/09/08 21:37:46 soyt Exp $
+/* $Id: palemu.c,v 1.9 2005/07/30 10:58:26 cegger Exp $
 ******************************************************************************
 
    Display-palemu: palette emulation on true-color modes
@@ -25,15 +25,14 @@
 ******************************************************************************
 */
 
+#include "config.h"
+#include <ggi/display/palemu.h>
+#include <ggi/internal/ggi_debug.h>
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
-#include "config.h"
-#include <ggi/internal/ggi-dl.h>
-
-#include <ggi/display/palemu.h>
-
 
 /**************************************************
  ***
@@ -44,8 +43,8 @@
 
 static void blitter_1(ggi_palemu_priv *priv, void *dest, void *src, int w)
 {
-	uint8 *s = (uint8 *) src;
-	uint8 *d = (uint8 *) dest;
+	uint8_t *s = (uint8_t *) src;
+	uint8_t *d = (uint8_t *) dest;
 
 	for (; w > 0; w--) {
 		*d++ = priv->lookup[*s++];
@@ -54,8 +53,8 @@ static void blitter_1(ggi_palemu_priv *priv, void *dest, void *src, int w)
 
 static void blitter_2(ggi_palemu_priv *priv, void *dest, void *src, int w)
 {
-	uint8  *s = (uint8  *) src;
-	uint16 *d = (uint16 *) dest;
+	uint8_t  *s = (uint8_t  *) src;
+	uint16_t *d = (uint16_t *) dest;
 
 	for (; w > 0; w--) {
 		*d++ = priv->lookup[*s++];
@@ -64,8 +63,8 @@ static void blitter_2(ggi_palemu_priv *priv, void *dest, void *src, int w)
 
 static void blitter_3(ggi_palemu_priv *priv, void *dest, void *src, int w)
 {
-	uint8 *s = (uint8 *) src;
-	uint8 *d = (uint8 *) dest;
+	uint8_t *s = (uint8_t *) src;
+	uint8_t *d = (uint8_t *) dest;
 
 	for (; w > 0; w--) {
 		ggi_pixel pix = priv->lookup[*s++];
@@ -78,8 +77,8 @@ static void blitter_3(ggi_palemu_priv *priv, void *dest, void *src, int w)
 
 static void blitter_4(ggi_palemu_priv *priv, void *dest, void *src, int w)
 {
-	uint8  *s = (uint8  *) src;
-	uint32 *d = (uint32 *) dest;
+	uint8_t  *s = (uint8_t  *) src;
+	uint32_t *d = (uint32_t *) dest;
 
 	for (; w > 0; w--) {
 		*d++ = priv->lookup[*s++];
@@ -101,8 +100,8 @@ int _ggi_palemu_Transfer(ggi_visual *vis, int x, int y, int w, int h)
 {
 	ggi_palemu_priv *priv = PALEMU_PRIV(vis);
 	int old_r_frame = vis->r_frame_num;
-	uint8 src_buf[8192];
-	uint8 dest_buf[8192];
+	uint8_t src_buf[8192];
+	uint8_t dest_buf[8192];
 
 	priv->mem_opdraw->setreadframe(vis, vis->d_frame_num);
 	
@@ -152,22 +151,23 @@ int _ggi_palemu_Flush(ggi_visual *vis)
 
 int _ggi_palemu_Open(ggi_visual *vis)
 {
+	int rc;
 	ggi_palemu_priv *priv = PALEMU_PRIV(vis);
 
 
-	GGIDPRINT("display-palemu: Open %dx%d#%dx%d\n", LIBGGI_X(vis), 
+	DPRINT("display-palemu: Open %dx%d#%dx%d\n", LIBGGI_X(vis), 
 		LIBGGI_Y(vis), LIBGGI_VIRTX(vis), LIBGGI_VIRTY(vis));
 
 
 	/* set the parent mode */
 	
-	if (ggiSetMode(priv->parent, &priv->mode) < 0) {
-
-		GGIDPRINT("display-palemu: Couldn't set parent mode.\n");
-		return -1;
+	rc = ggiSetMode(priv->parent, &priv->mode);
+	if (rc < 0) {
+		DPRINT("display-palemu: Couldn't set parent mode.\n");
+		return rc;
 	}
 
-	GGIDPRINT("display-palemu: parent is %d/%d\n",
+	DPRINT("display-palemu: parent is %d/%d\n",
 		GT_DEPTH(priv->mode.graphtype), 
 		GT_SIZE(priv->mode.graphtype));
 	
@@ -189,9 +189,9 @@ int _ggi_palemu_Open(ggi_visual *vis)
 		break;
 		
 	default:
-		GGIDPRINT("Unsupported pixel size '%d'.\n",
+		DPRINT("Unsupported pixel size '%d'.\n",
 			GT_SIZE(priv->mode.graphtype));
-		return -1;
+		return GGI_ENOMATCH;
 	}
 
 
