@@ -1,4 +1,4 @@
-/* $Id: TIvisual.h,v 1.3 2002/09/08 21:37:47 soyt Exp $
+/* $Id: TIvisual.h,v 1.6 2004/10/31 14:25:04 cegger Exp $
  *
  * Copyright 1998 MenTaLguY - mentalg@geocities.com
  *
@@ -20,9 +20,6 @@
    CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  */
 
-#include <stdlib.h>
-#include <stdio.h>
-
 #include "config.h"
 #include <ggi/internal/ggi-dl.h>
 
@@ -65,6 +62,8 @@ struct TIhooks {
 	ggi_coord physz;
 };
 
+#define TERMINFO_PRIV(vis)	((struct TIhooks *)LIBGGI_PRIVATE(vis))
+
 /* Prototypes
  */
 
@@ -87,3 +86,4 @@ giifunc_eventpoll	GII_terminfo_eventpoll;
 giifunc_sendevent	GII_terminfo_sendevent;
 
 extern int paint_ncurses_window(ggi_visual *, WINDOW *, int, int);
+extern void _GGI_terminfo_freedbs(ggi_visual *);
