@@ -1,4 +1,4 @@
-/* $Id: mode.c,v 1.2 2002/10/27 18:26:25 skids Exp $
+/* $Id: mode.c,v 1.15 2005/09/19 18:46:44 cegger Exp $
 ******************************************************************************
 
    Display-trueemu : mode management
@@ -26,12 +26,14 @@
 ******************************************************************************
 */
 
+#include "config.h"
+#include <ggi/display/trueemu.h>
+#include <ggi/internal/ggi_debug.h>
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
-#include <ggi/display/trueemu.h>
-
 #include "../common/pixfmt-setup.inc"
 #include "../common/gt-auto.inc"
 
@@ -48,7 +50,7 @@ static void _GGI_trueemu_freedbs(ggi_visual *vis)
 
 int GGI_trueemu_getapi(ggi_visual *vis, int num, char *apiname, char *arguments)
 {
-	strcpy(arguments, "");
+	*arguments = '\0';
 
 	switch(num) {
 
@@ -58,16 +60,20 @@ int GGI_trueemu_getapi(ggi_visual *vis, int num, char *apiname, char *arguments)
 	case 1: strcpy(apiname, "generic-stubs");
 		return 0;
 
-	case 2: sprintf(apiname, "generic-linear-%d%s",
+	case 2: sprintf(apiname, "generic-linear-%u%s",
 			GT_SIZE(LIBGGI_GT(vis)),
 			(LIBGGI_GT(vis) & GT_SUB_HIGHBIT_RIGHT) ? "-r" : "");
 		return 0;
 
 	case 3: strcpy(apiname, "generic-color");
 		return 0;
+		
+	case 4: strcpy(apiname, "generic-pseudo-stubs");
+		sprintf(arguments, "%p", (void *)TRUEEMU_PRIV(vis)->parent);
+		return 0;
 	}
 
-	return -1;
+	return GGI_ENOMATCH;
 }
 
 static int do_dbstuff(ggi_visual *vis)
@@ -81,9 +87,9 @@ static int do_dbstuff(ggi_visual *vis)
 	}
 	priv->frame_size = LIBGGI_FB_SIZE(LIBGGI_MODE(vis));
 	priv->fb_size = priv->frame_size * LIBGGI_MODE(vis)->frames;
-	priv->fb_ptr  = malloc(priv->fb_size);
+	priv->fb_ptr  = malloc((size_t)(priv->fb_size));
 
-	GGIDPRINT("display-trueemu: fb=%p size=%d frame=%d\n", 
+	DPRINT("display-trueemu: fb=%p size=%d frame=%d\n", 
 		priv->fb_ptr, priv->fb_size, priv->frame_size);
 
 	if (priv->fb_ptr == NULL) {
@@ -92,7 +98,7 @@ static int do_dbstuff(ggi_visual *vis)
 	}
 
 	/* clear all frames */
-	memset(priv->fb_ptr, 0, priv->fb_size);
+	memset(priv->fb_ptr, 0, (size_t)(priv->fb_size));
 
 	/* set up pixel format */
 	memset(LIBGGI_PIXFMT(vis), 0, sizeof(ggi_pixelformat));
@@ -114,7 +120,7 @@ static int do_dbstuff(ggi_visual *vis)
 		buf->layout = blPixelLinearBuffer;
 
 		buf->buffer.plb.stride = 
-			(LIBGGI_VIRTX(vis) * GT_SIZE(LIBGGI_GT(vis)) + 7) / 8;
+			GT_ByPPP(LIBGGI_VIRTX(vis), LIBGGI_GT(vis));
 		buf->buffer.plb.pixelformat = LIBGGI_PIXFMT(vis);
 	}
 
@@ -124,7 +130,7 @@ static int do_dbstuff(ggi_visual *vis)
 static int do_setmode(ggi_visual *vis)
 {
 	ggi_trueemu_priv *priv = TRUEEMU_PRIV(vis);
-	char libname[256], libargs[256];
+	char libname[GGI_MAX_APILEN], libargs[GGI_MAX_APILEN];
 	int err, id;
 
 	_GGI_trueemu_freedbs(vis);
@@ -135,14 +141,15 @@ static int do_setmode(ggi_visual *vis)
 
 	/* load libraries */
 	for (id=1; GGI_trueemu_getapi(vis, id, libname, libargs) == 0; id++) {
-		err = _ggiOpenDL(vis, libname, libargs, NULL);
+		err = _ggiOpenDL(vis, _ggiGetConfigHandle(),
+				libname, libargs, NULL);
 		if (err) {
 			fprintf(stderr,"display-tryeeny: Error opening the "
 				"%s (%s) library.\n", libname, libargs);
 			return GGI_EFATAL;
 		}
 
-		GGIDPRINT("Success in loading %s (%s)\n", libname, libargs);
+		DPRINT("Success in loading %s (%s)\n", libname, libargs);
 	}
 
 	/* Backup the current 2D operations, and override them with our
@@ -191,7 +198,7 @@ int GGI_trueemu_setmode(ggi_visual *vis, ggi_mode *mode)
 	ggi_trueemu_priv *priv = TRUEEMU_PRIV(vis);
 	int err;
 
-	GGIDPRINT_MODE("display-trueemu: setmode %dx%d#%dx%dF%d[0x%02x]\n",
+	DPRINT_MODE("display-trueemu: setmode %dx%d#%dx%dF%d[0x%02x]\n",
 			mode->visible.x, mode->visible.y,
 			mode->virt.x, mode->virt.y, 
 			mode->frames, mode->graphtype);
@@ -213,20 +220,21 @@ int GGI_trueemu_setmode(ggi_visual *vis, ggi_mode *mode)
 	priv->mode.frames  = 1;
 
 	if ((err = do_setmode(vis)) != 0) {
-		GGIDPRINT_MODE("display-trueemu: setmode failed (%d).\n", err);
+		DPRINT_MODE("display-trueemu: setmode failed (%d).\n", err);
 		return err;
 	}
 
-	GGIDPRINT_MODE("display-trueemu: Attempting to setmode on parent "
+	DPRINT_MODE("display-trueemu: Attempting to setmode on parent "
 		"visual...\n");
 
 	if ((err = _ggi_trueemu_Open(vis)) != 0) {
 		return err;
 	}
 
+	MANSYNC_SETFLAGS(vis, LIBGGI_FLAGS(vis));
 	MANSYNC_cont(vis);
 
-	GGIDPRINT_MODE("display-trueemu: setmode succeeded.\n");
+	DPRINT_MODE("display-trueemu: setmode succeeded.\n");
 
 	return 0;
 }
@@ -237,7 +245,7 @@ int GGI_trueemu_checkmode(ggi_visual *vis, ggi_mode *mode)
 	ggi_mode par_mode;
 	int tmperr, err = 0;
 	
-	GGIDPRINT_MODE("display-trueemu: checkmode %dx%d#%dx%dF%d[0x%02x]\n",
+	DPRINT_MODE("display-trueemu: checkmode %dx%d#%dx%dF%d[0x%02x]\n",
 		       mode->visible.x, mode->visible.y,
 		       mode->virt.x, mode->virt.y,
 		       mode->frames, mode->graphtype);
@@ -310,7 +318,7 @@ int GGI_trueemu_checkmode(ggi_visual *vis, ggi_mode *mode)
 	mode->dpp     = par_mode.dpp;
 	mode->size    = par_mode.size;
 
-	GGIDPRINT_MODE("display-trueemu: upgraded to %dx%d#%dx%dF%d[0x%02x]\n",
+	DPRINT_MODE("display-trueemu: upgraded to %dx%d#%dx%dF%d[0x%02x]\n",
 			mode->visible.x, mode->visible.y,
 			mode->virt.x, mode->virt.y,
 			mode->frames, mode->graphtype);
@@ -321,11 +329,11 @@ int GGI_trueemu_checkmode(ggi_visual *vis, ggi_mode *mode)
 int GGI_trueemu_getmode(ggi_visual *vis, ggi_mode *mode)
 {
 	if ((vis == NULL) || (mode == NULL) || (LIBGGI_MODE(vis) == NULL)) {
-		GGIDPRINT("display-trueemu: vis/mode == NULL\n");
-		return -1;
+		DPRINT("display-trueemu: vis/mode == NULL\n");
+		return GGI_ENOSPACE;
 	}
 	
-	GGIDPRINT("display-trueemu: getmode.\n");
+	DPRINT("display-trueemu: getmode.\n");
 
 	memcpy(mode, LIBGGI_MODE(vis), sizeof(ggi_mode));
 
