@@ -1,4 +1,4 @@
-/* $Id: trueemu.c,v 1.1.1.1 2001/05/12 23:02:40 cegger Exp $
+/* $Id: trueemu.c,v 1.7 2005/06/09 18:20:36 cegger Exp $
 ******************************************************************************
 
    Display-trueemu : truecolor emulation library.
@@ -25,12 +25,11 @@
 ******************************************************************************
 */
 
-#include <stdio.h>
-#include <stdlib.h>
-
-#include <ggi/internal/ggi-dl.h>
-
+#include "config.h"
 #include <ggi/display/trueemu.h>
+#include <ggi/internal/ggi_debug.h>
+
+#include <string.h>
 
 
 /**************************************************
@@ -139,7 +138,7 @@ static ggi_color black = { 0,0,0 };
  **************************************************/
 
 
-static int calc_default_flags(int flags, int graphtype)
+static int calc_default_flags(int flags, ggi_graphtype graphtype)
 {
 	int default_dither  = TRUEEMU_F_DITHER_4;
 	int default_palette = TRUEEMU_F_RGB;
@@ -178,7 +177,7 @@ static void load_332_palette(ggi_color *colormap)
 	for (g = 0; g < 8; g++)
 	for (b = 0; b < 4; b++) {
 
-		int index = (r << 5) | (g << 2) | b;
+		int _index = (r << 5) | (g << 2) | b;
 
 		ggi_color col;
 
@@ -186,7 +185,7 @@ static void load_332_palette(ggi_color *colormap)
 		col.g = g * 0xffff / (8-1);
 		col.b = b * 0xffff / (4-1);
 
-		colormap[index] = col;
+		memcpy(&colormap[_index], &col, sizeof(ggi_color));
 	}
 }
 
@@ -199,7 +198,7 @@ static void load_cube_palette(ggi_color *colormap)
 	for (g = 0; g < CUBE_LEVELS; g++)
 	for (b = 0; b < CUBE_LEVELS; b++) {
 
-		int index = (r * CUBE_LEVELS + g) * CUBE_LEVELS + b;
+		int _index = (r * CUBE_LEVELS + g) * CUBE_LEVELS + b;
 
 		ggi_color col;
 
@@ -207,14 +206,14 @@ static void load_cube_palette(ggi_color *colormap)
 		col.g = g * 0xffff / (CUBE_LEVELS-1);
 		col.b = b * 0xffff / (CUBE_LEVELS-1);
 
-		colormap[index] = col;
+		memcpy(&colormap[_index], &col, sizeof(ggi_color));
 	}
 }
 
 
 static void load_pastel_palette(ggi_color *colormap)
 {
-	int index;
+	int _index;
 	int pastel;
 
 
@@ -231,7 +230,7 @@ static void load_pastel_palette(ggi_color *colormap)
 		int tg = pastel_array[pastel].color.g;
 		int tb = pastel_array[pastel].color.b;
 
-		index = 1 + pastel * PASTEL_LEVELS;
+		_index = 1 + pastel * PASTEL_LEVELS;
 
 		for (pl=1; pl <= PASTEL_LEVELS; pl++) {
 
@@ -241,7 +240,7 @@ static void load_pastel_palette(ggi_color *colormap)
 			col.g = pl * tg / PASTEL_LEVELS;
 			col.b = pl * tb / PASTEL_LEVELS;
 
-			colormap[index++] = col;
+			memcpy(&colormap[_index++], &col, sizeof(ggi_color));
 		}
 	}
 }
@@ -249,10 +248,10 @@ static void load_pastel_palette(ggi_color *colormap)
 
 static void load_col16_palette(ggi_color *colormap)
 {
-	int index;
+	int i;
 
-	for (index=0; index < 16; index++) {
-		colormap[index] = col16_palette[index];
+	for (i=0; i < 16; i++) {
+		colormap[i] = col16_palette[i];
 	}
 }
 
@@ -265,7 +264,7 @@ static void load_121_palette(ggi_color *colormap)
 	for (g = 0; g < 4; g++)
 	for (b = 0; b < 2; b++) {
 
-		int index = (r << 3) | (g << 1) | b;
+		int _index = (r << 3) | (g << 1) | b;
 
 		ggi_color col;
 
@@ -273,7 +272,7 @@ static void load_121_palette(ggi_color *colormap)
 		col.g = g * 0xffff / (4-1);
 		col.b = b * 0xffff / (2-1);
 
-		colormap[index] = col;
+		memcpy(&colormap[_index], &col, sizeof(ggi_color));
 	}
 }
 
@@ -555,7 +554,7 @@ static void calc_pastel_dither(ggi_trueemu_priv *priv, int shift)
 	for (r=0; r < 32; r++) {
 	for (g=0; g < 32; g++) {
 	for (b=0; b < 32; b++) {
-		int index = (r << 10) | (g << 5) | b;
+		int _index = (r << 10) | (g << 5) | b;
 		int tr = r * 0xff / 31;
 		int tg = g * 0xff / 31;
 		int tb = b * 0xff / 31;
@@ -565,7 +564,7 @@ static void calc_pastel_dither(ggi_trueemu_priv *priv, int shift)
 		for (n=0; n < num; n++) {
 			int pl = (((max * bands) >> 5) + n) >> shift;
 
-			priv->T[index][n] = (pl == 0) ? 0 :
+			priv->T[_index][n] = (pl == 0) ? 0 :
 				1 + (pastel * PASTEL_LEVELS) + (pl - 1);
 		}
 	}
@@ -592,14 +591,14 @@ static void calc_col16_dither(ggi_trueemu_priv *priv, int shift)
 	for (g=0; g < 32; g++) {
 	for (b=0; b < 32; b++) {
 	for (n=0; n < num; n++) {
-		int index = (r << 10) | (g << 5) | b;
+		int _index = (r << 10) | (g << 5) | b;
 		int max = MAX(r, MAX(g, b));
 		int tr,tg,tb;
 		int cr,cg,cb;
 		int pl;
 
 		if (max == 0) {
-			priv->T[index][n] = 0;
+			priv->T[_index][n] = 0;
 			continue;
 		}
 
@@ -612,7 +611,7 @@ static void calc_col16_dither(ggi_trueemu_priv *priv, int shift)
 
 			pl = (((max * green_bands) >> 5) + n) >> shift;
 
-			priv->T[index][n] = col16_greens[pl];
+			priv->T[_index][n] = col16_greens[pl];
 			continue;
 		}
 
@@ -622,7 +621,7 @@ static void calc_col16_dither(ggi_trueemu_priv *priv, int shift)
 
 			pl = (((max * brown_bands) >> 5) + n) >> shift;
 
-			priv->T[index][n] = col16_browns[pl];
+			priv->T[_index][n] = col16_browns[pl];
 			continue;
 		}
 
@@ -633,7 +632,7 @@ static void calc_col16_dither(ggi_trueemu_priv *priv, int shift)
 
 			pl = (((max * grey_bands) >> 5) + n) >> shift;
 
-			priv->T[index][n] = col16_greys[pl];
+			priv->T[_index][n] = col16_greys[pl];
 			continue;
 		}
 
@@ -642,7 +641,7 @@ static void calc_col16_dither(ggi_trueemu_priv *priv, int shift)
 		cg = (((g * cube_bands) >> 5) + n) >> shift;
 		cb = (((b * cube_bands) >> 5) + n) >> shift;
 
-		priv->T[index][n] = ((cr*2) + cg)*2 + cb;
+		priv->T[_index][n] = ((cr*2) + cg)*2 + cb;
 	}
 	}
 	}
@@ -897,7 +896,7 @@ int _ggi_trueemu_Close(ggi_visual *vis)
 int _ggi_trueemu_Open(ggi_visual *vis)
 {
 	ggi_trueemu_priv *priv = TRUEEMU_PRIV(vis);
-	int bufsize;
+	size_t bufsize;
 	int err;
 
 	/* Free any previously allocated resources. */
@@ -912,11 +911,11 @@ int _ggi_trueemu_Open(ggi_visual *vis)
 	/* Set the parent mode */
 	err = ggiSetMode(priv->parent, &priv->mode);
 	if (err < 0) {
-		GGIDPRINT_MODE("display-trueemu: Couldn't set parent mode.\n");
+		DPRINT_MODE("display-trueemu: Couldn't set parent mode.\n");
 		return err;
 	}
 
-	GGIDPRINT_MODE("display-trueemu: parent is %d/%d\n",
+	DPRINT_MODE("display-trueemu: parent is %d/%d\n",
 		       GT_DEPTH(priv->mode.graphtype),
 		       GT_SIZE(priv->mode.graphtype));
 
