@@ -1,4 +1,4 @@
-/* $Id: patchlib.c,v 1.1.1.1 2001/05/12 23:00:38 cegger Exp $
+/* $Id: patchlib.c,v 1.4 2005/06/20 06:27:41 cegger Exp $
 ******************************************************************************
 
    Alter the path to the config file in binary libggi
@@ -25,23 +25,31 @@
 ******************************************************************************
 */
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <unistd.h>
-#include <fcntl.h> 
-#include <sys/stat.h>
-
 #include "config.h"
 
-void print_usage(char *fname)
+#include <stdio.h>
+#include <stdlib.h>
+
+#ifdef HAVE_STRING_H
+#include <string.h>
+#endif
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+#ifdef HAVE_FCNTL_H
+#include <fcntl.h> 
+#endif
+#include <sys/stat.h>
+
+
+static void print_usage(char *fname)
 {
         fprintf(stderr, "Usage : %s libggi.so-filename [new_path_to_patch_in]\n", fname);
         fprintf(stderr, "        Output will be located in libggi.so.patched\n");
 }
 
 static char tag[]=GGIPATHTAG;
-static char *newname = "libggi.so.patched";
+static const char *newname = "libggi.so.patched";
 
 int main(int argc, char *argv[])
 {
