@@ -1,4 +1,4 @@
-/* $Id: cbconsist.c,v 1.7.2.2 2004/05/16 15:56:37 cegger Exp $
+/* $Id: cbconsist.c,v 1.18 2005/07/30 11:58:39 cegger Exp $
 ******************************************************************************
 
    This is a consistency-test and benchmark application for LibGGI
@@ -26,13 +26,15 @@
  *    and to throw out bad marks from task scheduling.
  */
 
+#include "config.h"
+#include <ggi/ggi.h>
+#include <ggi/gg.h> 
+#include <ggi/internal/gg_replace.h>
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
-
-#include <ggi/ggi.h>
-
 #ifdef HAVE_GETOPT_H
 # include <getopt.h>
 #endif
@@ -45,7 +47,7 @@ typedef struct cbcstate_s cbcstate;
 
 struct cbcstate_s {
 	ggi_visual_t svis, dvis;
-	char *svisstr, *dvisstr;
+	const char *svisstr, *dvisstr;
 	ggi_mode smode, dmode;
 	ggi_pixel sblack, dblack;
 	int flags;
@@ -102,7 +104,7 @@ static ggi_pixel cbconsist(cbcstate * s)
 	count = 0xffffffff >> (32 - GT_SIZE(s->smode.graphtype));
 
 	do {
-		uint32 num;
+		uint32_t num;
 		ggi_pixel curr;
 
 		num = box.x * box.y;
@@ -220,7 +222,7 @@ static void cbtime(cbcstate * s)
  * possible pixel formats.
  */
 #define MAX_MEMVIS_FMTS 8
-char *memvis_fmts[MAX_MEMVIS_FMTS] = {
+static const char *memvis_fmts[MAX_MEMVIS_FMTS] = {
 	"memory:-pixfmt=r5g6b5",
 	"memory:-pixfmt=r4g4b4p4",
 	"memory:-pixfmt=b5g6r5",
@@ -231,7 +233,7 @@ char *memvis_fmts[MAX_MEMVIS_FMTS] = {
 	"memory:-pixfmt=p8r8g8b8"
 };
 
-ggi_graphtype memvis_gts[MAX_MEMVIS_FMTS] = {
+static ggi_graphtype memvis_gts[MAX_MEMVIS_FMTS] = {
 	GT_16BIT,
 	GT_16BIT,
 	GT_16BIT,
@@ -242,7 +244,7 @@ ggi_graphtype memvis_gts[MAX_MEMVIS_FMTS] = {
 	GT_32BIT
 };
 
-static int mkmemvis(int i, char **str,
+static int mkmemvis(int i, const char **str,
 	     ggi_visual_t * vis, ggi_mode * mode, ggi_pixel * black)
 {
 	ggi_color color;
@@ -281,7 +283,7 @@ static int mkmemvis(int i, char **str,
 	return i;
 }
 
-char optstring[] = "s:d:haftc";
+static const char optstring[] = "s:d:haftc";
 
 int main(int argc, char **argv)
 {
