@@ -1,4 +1,4 @@
-/* $Id: findleaks.c,v 1.2 2002/10/25 21:23:15 cegger Exp $
+/* $Id: findleaks.c,v 1.12 2005/06/09 18:46:12 cegger Exp $
 ******************************************************************************
 
    Helps to find memory leaks in LibGGI and targets.
@@ -15,48 +15,49 @@
 ******************************************************************************
 */
 
+#include "config.h"
+#include <ggi/ggi.h>
 #include <stdio.h>
 #include <stdlib.h>
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
 #include <fcntl.h>
-#include <ggi/ggi.h>
 
 #define _get_ggi_alloced() 0
 
-ggi_visual_t vis;
+static ggi_visual_t vis;
 
-void
-wait4key(void)
+#if 0
+static void wait4key(void)
 {
 	getchar();
 }
+#endif
 
-void
-err(char *s,...)
+static void err(const char *s,...)
 {
 	printf("Fatal error: ");
 	printf(s);
 	exit(1);
 }
 
-#ifdef __linux__
-int pid;
-char statpath[1024];
+#if defined(__linux__ ) || defined(__CYGWIN__)
+static int pid;
+static char statpath[1024];
 
-void
-fill_info(void)
+static void fill_info(void)
 {
 	pid = getpid();
 	sprintf(statpath, "/proc/%d/status", pid);
 }
 
-long
-get_size(void)
+static long get_size(void)
 {
 	long size=0;
 	FILE *fil;
 	char buf[1024];
-	
+
 	if ((fil = fopen(statpath, "r")) == NULL) {
 		err("opening %s\n", statpath);
 	}
@@ -68,8 +69,8 @@ get_size(void)
 }
 
 
-void
-inform_mem(char *info, long prev, long now, long allocprev, long allocnow)
+static void
+inform_mem(const char *info, long prev, long now, long allocprev, long allocnow)
 {
 	printf(info);
 	printf("Used before   : %6ld, after: %6ld,  Change: %6ld\n",
@@ -82,22 +83,22 @@ inform_mem(char *info, long prev, long now, long allocprev, long allocnow)
 
 #else
 
-void
-fill_info(void)
+static void fill_info(void)
 {
+	fprintf(stderr, "fill_info(): not implemented for your platform\n");
 }
 
-long
-get_size(void)
+static long get_size(void)
 {
+	fprintf(stderr, "get_size(): not implemented for your platform\n");
 	return 0;
 }
 
-void
-inform_mem(char *info, long prev, long now, long allocprev, long allocnow)
+static void inform_mem(const char *info, long prev, long now, long allocprev, long allocnow)
 {
 	printf(info);
-	printf("Check memory stats now!\n");
+	fprintf(stderr, "inform_mem(): not implemented for your platform\n");
+	printf("Check memory stats now, then press any key\n");
 	fflush(stdout);
 	fflush(stderr);
 	getchar();
