@@ -1,4 +1,4 @@
-/* $Id: menu.h,v 1.1.1.1 2001/05/12 23:03:51 cegger Exp $
+/* $Id: menu.h,v 1.3 2005/07/31 15:30:41 soyt Exp $
 ******************************************************************************
 
    Universal menu for ggi
@@ -11,7 +11,7 @@
    warranties of merchantability or fitness for a particular purpose.
    Use it at your own risk. the author is not responsible for any damage
    or consequences raised by use or inability to use this program.
- 
+
 ******************************************************************************
 */
 
@@ -23,7 +23,7 @@
 
 #define MAXENTRIES 9
 /* indexes 0 .. 8 but keys 1 ..9.
-** any good solution for this??  
+** any good solution for this??
 */
 
 struct menuentry {
@@ -35,7 +35,7 @@ struct menuentry {
 
 struct menu {
 	struct window w;
-	 
+
 	int lastentry ;                      /* number of last used entry */
 	struct menuentry entry[MAXENTRIES] ;
 
@@ -54,17 +54,17 @@ struct menu {
 	int entrylineskip;
 	int lastlineskip;
 
-	char * toptext;
+	const char * toptext;
 	int toptextx;
 	int toptexty;
 	ggi_color toptextcolor;
 
-	char * bottomtext;
+	const char * bottomtext;
 	int bottomtextx;
 	int bottomtexty;
 	ggi_color bottomtextcolor;
 
-      
+
 
 	int titlex, titley;    /* automatically calculated ! */
 
