@@ -1,4 +1,4 @@
-/* $Id: window.h,v 1.1.1.1 2001/05/12 23:03:53 cegger Exp $
+/* $Id: window.h,v 1.3 2005/07/31 15:30:41 soyt Exp $
 ******************************************************************************
 
    Universal window for LibGGI
@@ -11,7 +11,7 @@
    warranties of merchantability or fitness for a particular purpose.
    Use it at your own risk. the author is not responsible for any damage
    or consequences raised by use or inability to use this program.
- 
+
 ******************************************************************************
 */
 
@@ -26,14 +26,14 @@ struct window {
 	int xorigin;                 /* upper left corner */
 	int yorigin;                 /* relative to visual */
 	int xsize;                   /* graph width       */
-	int ysize;                   
+	int ysize;
 
 	ggi_color backgroundcolor;
-	
+
 	int borderwidth;             /* coloured border with # pixels */
 	ggi_color bordercolor;
 
-	char * title ;
+	const char *title ;
 	ggi_color titlecolor;
 };
 
